from copy import copy

from django.contrib import admin
from edc_action_item import action_fields, action_fieldset_tuple
from edc_model_admin import audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from ..forms import EndOfStudyForm


class EndOfStudyModelAdminMixin(ModelAdminSubjectDashboardMixin):

    form = EndOfStudyForm

    additional_instructions = (
        "Note: if the patient is deceased, complete the Death Report "
        "before completing this form. "
    )

    fieldsets = (
        [
            "Part 1:",
            {
                "fields": (
                    "subject_identifier",
                    "offschedule_datetime",
                    "offschedule_reason",
                    "other_offschedule_reason",
                    "ltfu_date",
                    "death_date",
                    "comment",
                )
            },
        ],
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    list_display = (
        "subject_identifier",
        "dashboard",
        "offschedule_datetime",
        "tracking_identifier",
        "action_identifier",
    )

    list_filter = ("offschedule_datetime",)

    radio_fields = {"offschedule_reason": admin.VERTICAL}

    search_fields = ("subject_identifier", "action_identifier", "tracking_identifier")

    def get_readonly_fields(self, request, obj=None):
        fields = super().get_readonly_fields(request, obj)
        action_flds = copy(list(action_fields))
        fields = list(action_flds) + list(fields)
        return fields



from django import forms
from edc_action_item.forms.action_item_form_mixin import ActionItemFormMixin
from edc_form_validators.form_validator_mixin import FormValidatorMixin
from edc_ltfu.modelform_mixins import LossToFollowupFormValidator
from edc_sites.forms import SiteModelFormMixin

from ..models import LossToFollowup


class LossToFollowupForm(
    SiteModelFormMixin, FormValidatorMixin, ActionItemFormMixin, forms.ModelForm
):

    form_validator_cls = LossToFollowupFormValidator

    subject_identifier = forms.CharField(
        label="Subject Identifier",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    class Meta:
        model = LossToFollowup
        fields = "__all__"



from multisite import SiteID

from .defaults import *  # noqa

print(f"Settings file {__file__}")  # noqa

SITE_ID = SiteID(default=1)
EDC_SITES_UAT_DOMAIN = False
AUTO_CREATE_KEYS = False
ALLOWED_HOSTS = [
    "amana.tz.mocca.clinicedc.org",
    "bunju.tz.mocca.clinicedc.org",
    "hindu-mandal.tz.mocca.clinicedc.org",
    "kisugu.ug.mocca.clinicedc.org",
    "kiswa.ug.mocca.clinicedc.org",
    "mkuranga.tz.mocca.clinicedc.org",
    "mulago.ug.mocca.clinicedc.org",
    "mwananyamala.tz.mocca.clinicedc.org",
    "ndejje.ug.mocca.clinicedc.org",
    "wakiso.ug.mocca.clinicedc.org",
    "localhost",
]
AUTO_CREATE_KEYS = False



# Generated by Django 3.0.9 on 2020-12-16 01:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_subject", "0004_remove_reasonforvisit_health_services"),
        ("mocca_lists", "0001_initial"),
    ]

    operations = [
        migrations.DeleteModel(
            name="HealthServices",
        ),
    ]



from django import forms

from ..models import DailyClosingLog


class DailyClosingLogForm(forms.ModelForm):
    def clean(self):
        cleaned_data = super().clean()
        attended = cleaned_data.get("attended")
        approached = cleaned_data.get("approached")
        agreed_to_screen = cleaned_data.get("agreed_to_screen")
        if attended is not None and approached is not None:
            if approached > attended:
                raise forms.ValidationError(
                    {
                        "approached": (
                            "Invalid. Number approached cannot be greater "
                            "than number attended"
                        )
                    }
                )
        if approached is not None and agreed_to_screen is not None:
            if agreed_to_screen > approached:
                raise forms.ValidationError(
                    {
                        "agreed_to_screen": (
                            "Invalid. Number who agreed to be screened cannot "
                            "be greater than number approached"
                        )
                    }
                )
        return cleaned_data

    class Meta:
        model = DailyClosingLog
        fields = "__all__"



from .autocomplete_admin import RxAdmin
from .blood_results_lipid_admin import BloodResultsLipidAdmin
from .cd4_result_admin import Cd4ResultAdmin
from .chol_initial_review_admin import CholInitialReviewAdmin
from .chol_medication_adherence_admin import CholMedicationAdherenceAdmin
from .chol_review_admin import CholReviewAdmin
from .clinical_review_admin import ClinicalReviewAdmin
from .clinical_review_baseline_admin import ClinicalReviewBaselineAdmin
from .complications_baseline_admin import ComplicationsBaselineAdmin
from .complications_followup_admin import ComplicationsFollowupAdmin
from .dm_initial_review_admin import DmInitialReviewAdmin
from .dm_medication_adherence_admin import DmMedicationAdherenceAdmin
from .dm_review_admin import DmReviewAdmin
from .drug_refill_chol_admin import DrugRefillCholAdmin
from .drug_refill_dm_admin import DrugRefillDmAdmin
from .drug_refill_hiv_admin import DrugRefillHivAdmin
from .drug_refill_htn_admin import DrugRefillHtnAdmin
from .family_history_admin import FamilyHistoryAdmin
from .glucose_baseline_admin import GlucoseBaselineAdmin
from .glucose_followup_admin import GlucoseFollowupAdmin
from .hiv_initial_review_admin import HivInitialReviewAdmin
from .hiv_medication_adherence_admin import HivMedicationAdherenceAdmin
from .hiv_review_admin import HivReviewAdmin
from .htn_initial_review_admin import HtnInitialReviewAdmin
from .htn_medication_adherence_admin import HtnMedicationAdherenceAdmin
from .htn_review_admin import HtnReviewAdmin
from .indicators_admin import IndicatorsAdmin
from .medications_admin import MedicationsAdmin
from .next_appointment_admin import NextAppointmentAdmin
from .other_baseline_data_admin import OtherBaselineDataAdmin
from .patient_health_admin import PatientHealthAdmin
from .subject_requisition_admin import SubjectRequisitionAdmin
from .subject_visit_admin import SubjectVisitAdmin
from .subject_visit_missed_admin import SubjectVisitMissedAdmin
from .viral_load_result_admin import ViralLoadResultAdmin



from edc_randomization.randomizer import Randomizer as Base
from edc_randomization.site_randomizers import site_randomizers


class Randomizer(Base):
    assignment_map = {"integration": 1}
    is_blinded_trial = False


site_randomizers.register(Randomizer)



from django.core.validators import (
    MaxLengthValidator,
    MaxValueValidator,
    MinLengthValidator,
    MinValueValidator,
    RegexValidator,
)
from django.db import models
from django_crypto_fields.fields import EncryptedCharField
from edc_constants.choices import YES_NO, YES_NO_NA
from edc_constants.constants import NO, NOT_APPLICABLE, YES
from edc_model.models import BaseUuidModel
from edc_screening.model_mixins import EligibilityModelMixin, ScreeningModelMixin
from edc_screening.screening_identifier import (
    ScreeningIdentifier as BaseScreeningIdentifier,
)
from edc_utils import get_utcnow

from mocca_lists.models import MoccaOriginalSites

from ..eligibility import MoccaScreeningEligibility
from ..mocca_original_sites import get_mocca_site_limited_to
from .mocca_register import MoccaRegister
from .model_mixins import CareModelMixin

PREG_YES_NO_NA = (
    (YES, "Yes"),
    (NO, "No"),
    (NOT_APPLICABLE, "Not Applicable"),
)


class ScreeningIdentifier(BaseScreeningIdentifier):
    template = "S{random_string}"


class SubjectScreening(
    CareModelMixin,
    ScreeningModelMixin,
    EligibilityModelMixin,
    BaseUuidModel,
):
    identifier_cls = ScreeningIdentifier

    eligibility_cls = MoccaScreeningEligibility

    subject_identifier = models.CharField(max_length=50, null=True, blank=True)

    screening_consent = models.CharField(
        verbose_name=(
            "Has the subject given his/her verbal consent "
            "to be screened for the MOCCA Extension study?"
        ),
        max_length=15,
        choices=YES_NO,
    )

    mocca_site = models.ForeignKey(
        MoccaOriginalSites,
        verbose_name="Original MOCCA site",
        on_delete=models.PROTECT,
        limit_choices_to=get_mocca_site_limited_to,
    )

    mocca_participant = models.CharField(
        verbose_name="Was the patient enrolled to the original MOCCA study?",
        max_length=25,
        choices=YES_NO,
        default=YES,
    )

    mocca_study_identifier = models.CharField(
        verbose_name="MOCCA (original) study identifier",
        unique=True,
        max_length=25,
        # validators=[
        #     RegexValidator(
        #         r"0[0-9]{1}\-0[0-9]{3}|[0-9]{6}",
        #         "Invalid format. Expected 12-3456 for UG, 123456 for TZ",
        #     )
        # ],
        help_text="Format must match original identifier. e.g. 12-3456 for UG, 123456 for TZ",
    )

    initials = EncryptedCharField(
        validators=[
            RegexValidator("[A-Z]{1,3}", "Invalid format"),
            MinLengthValidator(2),
            MaxLengthValidator(3),
        ],
        help_text=(
            "Use UPPERCASE letters only. May be 2 or 3 letters. "
            "Use `F`irst`L`ast or `L`ast`F`irst depending on the country custom. "
        ),
        blank=False,
    )

    birth_year = models.IntegerField(
        validators=[MinValueValidator(1900), MaxValueValidator(2002)],
    )

    mocca_register = models.OneToOneField(
        MoccaRegister,
        on_delete=models.PROTECT,
        null=True,
        verbose_name="MOCCA (original) register details",
    )

    willing_to_consent = models.CharField(
        verbose_name=(
            "Has the patient expressed willingness to participate "
            "in the `MOCCA extension` study"
        ),
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text="If No, complete refusal form.",
    )

    pregnant = models.CharField(
        verbose_name="Is the patient pregnant?",
        max_length=15,
        choices=PREG_YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    requires_acute_care = models.CharField(
        verbose_name="Does the patient require acute care including in-patient admission",
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    def save(self, *args, **kwargs):
        self.age_in_years = self.mocca_register.age_in_years
        self.birth_year = self.mocca_register.birth_year
        self.gender = self.mocca_register.gender
        self.initials = self.mocca_register.initials
        self.mocca_screening_identifier = self.mocca_register.mocca_screening_identifier
        self.mocca_site = self.mocca_register.mocca_site
        self.mocca_study_identifier = self.mocca_register.mocca_study_identifier
        self.eligibility_datetime = self.report_datetime
        super().save(*args, **kwargs)

    class Meta:
        verbose_name = "Subject Screening"
        verbose_name_plural = "Subject Screening"



from django.db import models
from edc_constants.choices import SMOKER_STATUS_SIMPLE, YES_NO
from edc_constants.constants import NOT_APPLICABLE
from edc_model import models as edc_models

from ..choices import ALCOHOL_CONSUMPTION, EDUCATION, EMPLOYMENT_STATUS, MARITAL_STATUS
from ..model_mixins import CrfModelMixin


class OtherBaselineData(CrfModelMixin, edc_models.BaseUuidModel):

    employment_status = models.CharField(
        verbose_name="What is the patient's employment status?",
        max_length=25,
        choices=EMPLOYMENT_STATUS,
    )

    employment_status_other = edc_models.OtherCharField()

    education = models.CharField(
        verbose_name="How much formal education does the patient have?",
        max_length=25,
        choices=EDUCATION,
    )

    marital_status = models.CharField(
        verbose_name="Personal status?",
        max_length=25,
        choices=MARITAL_STATUS,
    )

    smoking_status = models.CharField(
        verbose_name="Which of these options describes you",
        max_length=15,
        choices=SMOKER_STATUS_SIMPLE,
    )

    smoker_quit_ago = edc_models.DurationYMDField(
        verbose_name="If you used to smoke but stopped, how long ago did you stop",
        null=True,
        blank=True,
    )

    smoker_quit_estimated_date = models.DateField(
        verbose_name="Estimated date smoker quit?",
        null=True,
        editable=False,
    )

    alcohol = models.CharField(
        verbose_name="Do you drink alcohol?",
        max_length=15,
        choices=YES_NO,
    )

    alcohol_consumption = models.CharField(
        verbose_name="If yes, how often do you drink alcohol?",
        max_length=25,
        choices=ALCOHOL_CONSUMPTION,
        default=NOT_APPLICABLE,
    )

    def save(self, *args, **kwargs):
        if self.smoker_quit_ago:
            self.smoker_quit_estimated_date = edc_models.duration_to_date(
                self.smoker_quit_ago, self.report_datetime
            )
        super().save(*args, **kwargs)

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Other Baseline Data"
        verbose_name_plural = "Other Baseline Data"



from django import forms
from edc_constants.constants import NO
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_dx_review.utils import raise_if_clinical_review_does_not_exist
from edc_form_validators.form_validator import FormValidator

from ..models import HtnReview


class HtnReviewFormValidator(CrfFormValidatorMixin, FormValidator):
    def clean(self):
        raise_if_clinical_review_does_not_exist(self.cleaned_data.get("subject_visit"))
        self.validate_care_delivery()
        self.validate_bp_reading(
            "sys_blood_pressure",
            "dia_blood_pressure",
        )

    def validate_care_delivery(self) -> None:
        self.required_if(NO, field="care_delivery", field_required="care_delivery_other")

    def validate_bp_reading(self, sys_field, dia_field):
        if self.cleaned_data.get(sys_field) and self.cleaned_data.get(dia_field):
            if self.cleaned_data.get(sys_field) < self.cleaned_data.get(dia_field):
                raise forms.ValidationError(
                    {dia_field: "Systolic must be greater than diastolic."}
                )


class HtnReviewForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = HtnReviewFormValidator

    class Meta:
        model = HtnReview
        fields = "__all__"



from edc_constants.constants import NO, NOT_APPLICABLE, YES
from edc_metadata import NOT_REQUIRED, REQUIRED
from edc_metadata.metadata_rules import CrfRule, CrfRuleGroup, P, register
from edc_visit_tracking.constants import MISSED_VISIT

from mocca_subject.action_items import is_baseline


def func_glucose_baseline_required(visit=None, **kwargs):
    return is_baseline(visit)


def func_glucose_required(visit=None, **kwargs):
    return not is_baseline(visit) and visit.reason != MISSED_VISIT


@register()
class ClinicalReviewBaselineRuleGroup(CrfRuleGroup):

    hiv = CrfRule(
        predicate=P("hiv_dx", "eq", YES),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["hivinitialreview"],
    )

    dm = CrfRule(
        predicate=P("dm_dx", "eq", YES),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["dminitialreview"],
    )

    htn = CrfRule(
        predicate=P("htn_dx", "eq", YES),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["htninitialreview"],
    )

    chol = CrfRule(
        predicate=P("chol_dx", "eq", YES),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["cholinitialreview"],
    )

    class Meta:
        app_label = "mocca_subject"
        source_model = "mocca_subject.clinicalreviewbaseline"


@register()
class ClinicalReviewRuleGroup(CrfRuleGroup):

    hiv_dx = CrfRule(
        predicate=P("hiv_dx", "eq", YES),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["hivinitialreview"],
    )

    dm_dx = CrfRule(
        predicate=P("dm_dx", "eq", YES),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["dminitialreview"],
    )

    htn_dx = CrfRule(
        predicate=P("htn_dx", "eq", YES),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["htninitialreview"],
    )

    chol_dx = CrfRule(
        predicate=P("chol_dx", "eq", YES),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["cholinitialreview"],
    )

    hiv_test = CrfRule(
        predicate=P("hiv_test", "eq", NOT_APPLICABLE),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["hivreview"],
    )

    dm_test = CrfRule(
        predicate=P("dm_test", "eq", NOT_APPLICABLE),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["dmreview"],
    )

    htn_test = CrfRule(
        predicate=P("htn_test", "eq", NOT_APPLICABLE),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["htnreview"],
    )

    chol_test = CrfRule(
        predicate=P("chol_test", "eq", NOT_APPLICABLE),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["cholreview"],
    )

    complications = CrfRule(
        predicate=P("complications", "eq", YES),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["complicationsfollowup"],
    )

    class Meta:
        app_label = "mocca_subject"
        source_model = "mocca_subject.clinicalreview"


@register()
class GlucoseRuleGroup(CrfRuleGroup):

    gluc_baseline = CrfRule(
        predicate=func_glucose_baseline_required,
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["glucosebaseline"],
    )

    gluc_followup = CrfRule(
        predicate=func_glucose_required,
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["glucosefollowup"],
    )

    class Meta:
        app_label = "mocca_subject"
        source_model = "mocca_subject.subjectvisit"


@register()
class MedicationsRuleGroup(CrfRuleGroup):

    refill_hiv = CrfRule(
        predicate=P("refill_hiv", "eq", YES),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["drugrefillhiv"],
    )

    refill_dm = CrfRule(
        predicate=P("refill_dm", "eq", YES),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["drugrefilldm"],
    )

    refill_htn = CrfRule(
        predicate=P("refill_htn", "eq", YES),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["drugrefillhtn"],
    )

    refill_chol = CrfRule(
        predicate=P("refill_chol", "eq", YES),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["drugrefillchol"],
    )

    adherence_hiv = CrfRule(
        predicate=P("refill_hiv", "in", [YES, NO]),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["hivmedicationadherence"],
    )

    adherence_dm = CrfRule(
        predicate=P("refill_dm", "in", [YES, NO]),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["dmmedicationadherence"],
    )

    adherence_htn = CrfRule(
        predicate=P("refill_htn", "in", [YES, NO]),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["htnmedicationadherence"],
    )

    adherence_chol = CrfRule(
        predicate=P("refill_chol", "in", [YES, NO]),
        consequence=REQUIRED,
        alternative=NOT_REQUIRED,
        target_models=["cholmedicationadherence"],
    )

    class Meta:
        app_label = "mocca_subject"
        source_model = "mocca_subject.medications"



# Generated by Django 3.1.6 on 2021-02-28 20:19

from django.db import migrations, models
import edc_model.models.validators.date


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_subject', '0009_auto_20210228_2107'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='dminitialreview',
            name='glucose',
        ),
        migrations.RemoveField(
            model_name='dminitialreview',
            name='glucose_date',
        ),
        migrations.RemoveField(
            model_name='dminitialreview',
            name='glucose_fasted',
        ),
        migrations.RemoveField(
            model_name='dminitialreview',
            name='glucose_quantifier',
        ),
        migrations.RemoveField(
            model_name='dminitialreview',
            name='glucose_units',
        ),
        migrations.RemoveField(
            model_name='historicaldminitialreview',
            name='glucose',
        ),
        migrations.RemoveField(
            model_name='historicaldminitialreview',
            name='glucose_date',
        ),
        migrations.RemoveField(
            model_name='historicaldminitialreview',
            name='glucose_fasted',
        ),
        migrations.RemoveField(
            model_name='historicaldminitialreview',
            name='glucose_quantifier',
        ),
        migrations.RemoveField(
            model_name='historicaldminitialreview',
            name='glucose_units',
        ),
        migrations.AlterField(
            model_name='dminitialreview',
            name='glucose_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], help_text='If yes, complete the Glucose result form.', max_length=15, verbose_name='Has the patient had their glucose measured in the last few months?'),
        ),
        migrations.AlterField(
            model_name='dminitialreview',
            name='managed_by',
            field=models.CharField(choices=[('insulin', 'Insulin injections'), ('drugs', 'Oral drugs'), ('diet_lifestyle', 'Diet and lifestyle alone')], default='N/A', help_text='If insulin or oral drugs, diet and lifestyle is assumed.', max_length=25, verbose_name="How is the patient's diabetes managed?"),
        ),
        migrations.AlterField(
            model_name='historicaldminitialreview',
            name='glucose_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], help_text='If yes, complete the Glucose result form.', max_length=15, verbose_name='Has the patient had their glucose measured in the last few months?'),
        ),
        migrations.AlterField(
            model_name='historicaldminitialreview',
            name='managed_by',
            field=models.CharField(choices=[('insulin', 'Insulin injections'), ('drugs', 'Oral drugs'), ('diet_lifestyle', 'Diet and lifestyle alone')], default='N/A', help_text='If insulin or oral drugs, diet and lifestyle is assumed.', max_length=25, verbose_name="How is the patient's diabetes managed?"),
        ),
        migrations.AlterField(
            model_name='historicalhivinitialreview',
            name='arv_initiation_actual_date',
            field=models.DateField(blank=True, help_text='Calculated based on response to `arv_initiation_ago`', null=True, validators=[edc_model.models.validators.date.date_not_future], verbose_name='Date started antiretroviral therapy (ART)'),
        ),
        migrations.AlterField(
            model_name='hivinitialreview',
            name='arv_initiation_actual_date',
            field=models.DateField(blank=True, help_text='Calculated based on response to `arv_initiation_ago`', null=True, validators=[edc_model.models.validators.date.date_not_future], verbose_name='Date started antiretroviral therapy (ART)'),
        ),
    ]



from edc_list_data.model_mixins import ListModelMixin


class MoccaOriginalSites(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "MOCCA Original Site"
        verbose_name_plural = "MOCCA Original Sites"


class ArvDrugs(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Arv Drugs"
        verbose_name_plural = "Arv Drugs"


class ArvRegimens(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "ARV Regimens"
        verbose_name_plural = "ARV Regimens"


class ClinicServices(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Clinic Services"
        verbose_name_plural = "Clinic Services"


class Conditions(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Conditions"
        verbose_name_plural = "Conditions"


class DmTreatments(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Diabetes Treatments"
        verbose_name_plural = "Diabetes Treatments"


class HtnTreatments(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Hypertension Treatments"
        verbose_name_plural = "Hypertension Treatments"


class CholTreatments(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Cholesterol Treatments"
        verbose_name_plural = "Cholesterol Treatments"


class OffstudyReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Offstudy Reasons"
        verbose_name_plural = "Offstudy Reasons"


class NonAdherenceReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "NonAdherence Reasons"
        verbose_name_plural = "NonAdherence Reasons"


class RxModifications(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Treatment Modifications"
        verbose_name_plural = "Treatment Modifications"


class RxModificationReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Treatment Modification Reasons"
        verbose_name_plural = "Treatment Modification Reasons"


class ReasonsForTesting(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Reasons for Testing"
        verbose_name_plural = "Reasons for Testing"


class RefillConditions(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Refill Conditions"
        verbose_name_plural = "Refill Conditions"


class VisitReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Visit Reasons"
        verbose_name_plural = "Visit Reasons"


class SubjectVisitMissedReasons(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Subject Missed Visit Reasons"
        verbose_name_plural = "Subject Missed Visit Reasons"


class DrugPaySources(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Drug Payment Sources"
        verbose_name_plural = "Drug Payment Sources"


class TransportChoices(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Transport Choices"
        verbose_name_plural = "Transport Choices"


class DiagnosisLocations(ListModelMixin):
    class Meta(ListModelMixin.Meta):
        verbose_name = "Diagnosis Locations"
        verbose_name_plural = "Diagnosis Locations"



# Generated by Django 3.0.9 on 2020-12-04 20:39

import django_crypto_fields.fields.encrypted_char_field
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0014_auto_20201204_1949"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalmoccaregister",
            name="best_tel",
            field=models.CharField(
                choices=[
                    ("tel_1", "Tel/Mobile(1)"),
                    ("tel_2", "Tel/Mobile(2)"),
                    ("tel_3", "Tel/Mobile(3)"),
                ],
                max_length=15,
                null=True,
                verbose_name="Best Telephone / Mobile",
            ),
        ),
        migrations.AddField(
            model_name="historicalmoccaregister",
            name="tel_one",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Tel/Mobile(1)",
            ),
        ),
        migrations.AddField(
            model_name="historicalmoccaregister",
            name="tel_three",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Tel/Mobile(3)",
            ),
        ),
        migrations.AddField(
            model_name="historicalmoccaregister",
            name="tel_two",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Tel/Mobile(2)",
            ),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="best_tel",
            field=models.CharField(
                choices=[
                    ("tel_1", "Tel/Mobile(1)"),
                    ("tel_2", "Tel/Mobile(2)"),
                    ("tel_3", "Tel/Mobile(3)"),
                ],
                max_length=15,
                null=True,
                verbose_name="Best Telephone / Mobile",
            ),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="tel_one",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Tel/Mobile(1)",
            ),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="tel_three",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Tel/Mobile(3)",
            ),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="tel_two",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Tel/Mobile(2)",
            ),
        ),
    ]



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple
from edc_model_admin import SimpleHistoryAdmin

from ..admin_site import mocca_subject_admin
from ..forms import HivInitialReviewForm
from ..models import HivInitialReview
from .modeladmin_mixins import CrfModelAdminMixin


@admin.register(HivInitialReview, site=mocca_subject_admin)
class HivInitialReviewAdmin(CrfModelAdminMixin, SimpleHistoryAdmin):
    form = HivInitialReviewForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Diagnosis and Care",
            {
                "fields": (
                    "dx_ago",
                    "dx_date",
                    "dx_location",
                    "dx_location_other",
                    "receives_care",
                    "clinic",
                    "clinic_other",
                )
            },
        ),
        (
            "Monitoring and Treatment",
            {
                "fields": (
                    "arv_initiated",
                    "arv_initiation_ago",
                    "arv_initiation_actual_date",
                    "has_vl",
                    "vl",
                    "vl_quantifier",
                    "vl_date",
                    "has_cd4",
                    "cd4",
                    "cd4_date",
                ),
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "arv_initiated": admin.VERTICAL,
        "clinic": admin.VERTICAL,
        "crf_status": admin.VERTICAL,
        "has_cd4": admin.VERTICAL,
        "has_vl": admin.VERTICAL,
        "receives_care": admin.VERTICAL,
        "vl_quantifier": admin.VERTICAL,
        "dx_location": admin.VERTICAL,
    }



from django.conf import settings
from django.db import models
from edc_constants.choices import YES_NO, YES_NO_NA
from edc_constants.constants import NO, NOT_APPLICABLE
from edc_model import models as edc_models
from edc_model.utils import estimated_date_from_ago


class ReviewModelMixin(models.Model):

    care_delivery = models.CharField(
        verbose_name=(
            "Was care for this `condition` delivered in an integrated care clinic today?"
        ),
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text="Select `not applicable` if site was not selected for integrated care.",
    )

    care_delivery_other = models.TextField(
        verbose_name="If no, please explain", null=True, blank=True
    )

    class Meta:
        abstract = True


class ComplicationsBaselineModelMixin(models.Model):

    stroke = models.CharField(
        verbose_name="Stroke",
        max_length=25,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    stroke_ago = edc_models.DurationYMDField(
        verbose_name="If yes, how long ago",
        null=True,
        blank=True,
    )

    stroke_estimated_date = models.DateField(
        verbose_name="Estimated date of stroke",
        null=True,
        blank=True,
        editable=False,
    )

    # heart_attack
    heart_attack = models.CharField(
        verbose_name="Heart attack / heart failure",
        max_length=25,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    heart_attack_ago = edc_models.DurationYMDField(
        verbose_name="If yes, how long ago",
        null=True,
        blank=True,
    )

    heart_attack_estimated_date = models.DateField(
        verbose_name="Estimated date of heart attack / heart failure",
        null=True,
        blank=True,
        editable=False,
    )

    # renal
    renal_disease = models.CharField(
        verbose_name="Renal (kidney) disease",
        max_length=25,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    renal_disease_ago = edc_models.DurationYMDField(
        verbose_name="If yes, how long ago",
        null=True,
        blank=True,
    )

    renal_disease_estimated_date = models.DateField(
        verbose_name="Estimated date of renal_disease",
        null=True,
        blank=True,
        editable=False,
    )
    # vision
    vision = models.CharField(
        verbose_name="Vision problems (e.g. blurred vision)",
        max_length=25,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    vision_ago = edc_models.DurationYMDField(
        verbose_name="If yes, how long ago",
        null=True,
        blank=True,
    )

    vision_estimated_date = models.DateField(
        verbose_name="Estimated date of vision problems",
        null=True,
        blank=True,
        editable=False,
    )

    # numbness
    numbness = models.CharField(
        verbose_name="Numbness / burning sensation",
        max_length=25,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    numbness_ago = edc_models.DurationYMDField(
        verbose_name="If yes, how long ago",
        null=True,
        blank=True,
    )

    numbness_estimated_date = models.DateField(
        verbose_name="Estimated date of numbness",
        null=True,
        blank=True,
        editable=False,
    )

    # foot ulcers
    foot_ulcers = models.CharField(
        verbose_name="Foot ulcers",
        max_length=25,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    foot_ulcers_ago = edc_models.DurationYMDField(
        verbose_name="If yes, how long ago",
        null=True,
        blank=True,
    )

    foot_ulcers_estimated_date = models.DateField(
        verbose_name="Estimated date of foot ulcers",
        null=True,
        blank=True,
        editable=False,
    )
    #
    complications = models.CharField(
        verbose_name="Are there any other major complications to report?",
        max_length=25,
        choices=YES_NO,
        default=NO,
    )

    complications_other = models.TextField(
        null=True,
        blank=True,
        help_text="Please include dates",
    )

    def save(self, *args, **kwargs):
        complications = [
            "stroke",
            "heart_attack",
            "renal_disease",
            "vision",
            "numbness",
            "foot_ulcers",
        ]
        for complication in complications:
            setattr(
                self,
                f"{complication}_estimated_date",
                estimated_date_from_ago(self, f"{complication}_ago"),
            )
        super().save(*args, **kwargs)

    class Meta:
        abstract = True
        verbose_name = "Complications: Baseline"
        verbose_name_plural = "Complications: Baseline"


class ComplicationsFollowupMixin(models.Model):

    # stroke
    stroke = models.CharField(verbose_name="Stroke", max_length=25, choices=YES_NO)

    stroke_date = models.DateField(
        verbose_name="If yes, date",
        null=True,
        blank=True,
        help_text="If exact date not known, see SOP on how to estimate a date.",
    )

    # heart_attack
    heart_attack = models.CharField(
        verbose_name="Heart attack / heart failure",
        max_length=25,
        choices=YES_NO,
    )

    heart_attack_date = models.DateField(
        verbose_name="If yes, date",
        null=True,
        blank=True,
        help_text="If exact date not known, see SOP on how to estimate a date.",
    )

    # renal
    renal_disease = models.CharField(
        verbose_name="Renal (kidney) disease",
        max_length=25,
        choices=YES_NO,
    )

    renal_disease_date = models.DateField(
        verbose_name="If yes, date",
        null=True,
        blank=True,
        help_text="If exact date not known, see SOP on how to estimate a date.",
    )
    # vision
    vision = models.CharField(
        verbose_name="Vision problems (e.g. blurred vision)",
        max_length=25,
        choices=YES_NO,
    )

    vision_date = models.DateField(
        verbose_name="If yes, date",
        null=True,
        blank=True,
        help_text="If exact date not known, see SOP on how to estimate a date.",
    )

    # numbness
    numbness = models.CharField(
        verbose_name="Numbness / burning sensation",
        max_length=25,
        choices=YES_NO,
    )

    numbness_date = models.DateField(
        verbose_name="If yes, date",
        null=True,
        blank=True,
        help_text="If exact date not known, see SOP on how to estimate a date.",
    )

    # foot ulcers
    foot_ulcers = models.CharField(
        verbose_name="Foot ulcers",
        max_length=25,
        choices=YES_NO,
    )

    foot_ulcers_date = models.DateField(
        verbose_name="If yes, date",
        null=True,
        blank=True,
        help_text="If exact date not known, see SOP on how to estimate a date.",
    )
    #
    complications = models.CharField(
        verbose_name="Are there any other major complications to report?",
        max_length=25,
        choices=YES_NO,
        default=NO,
    )

    complications_other = models.TextField(
        null=True,
        blank=True,
        help_text="Please include dates",
    )

    class Meta:
        abstract = True
        verbose_name = "Complications: Followup"
        verbose_name_plural = "Complications: Followup"


class DrugRefillModelMixin(models.Model):

    rx_other = models.CharField(
        verbose_name="If other, please specify ...",
        max_length=150,
        null=True,
        blank=True,
    )

    rx_modified = models.CharField(
        verbose_name=(
            "Was the patient’s prescription changed "
            "at this visit compared with their prescription "
            "at the previous visit?"
        ),
        max_length=25,
        choices=YES_NO,
    )

    modifications = models.ManyToManyField(
        f"{settings.LIST_MODEL_APP_LABEL}.RxModifications",
        verbose_name="Which changes occurred?",
        blank=True,
    )

    modifications_other = models.CharField(
        verbose_name="If other, please specify ...",
        max_length=150,
        null=True,
        blank=True,
    )

    modifications_reason = models.ManyToManyField(
        f"{settings.LIST_MODEL_APP_LABEL}.RxModificationReasons",
        verbose_name="Why did the patient’s previous prescription change?",
        blank=True,
    )

    modifications_reason_other = models.CharField(
        verbose_name="If other, please specify ...",
        max_length=150,
        null=True,
        blank=True,
    )

    return_in_days = models.IntegerField(
        verbose_name=(
            "In how many days has the patient been asked "
            "to return to clinic for a drug refill?"
        )
    )

    class Meta:
        abstract = True


class HivMedicationsModelMixin(models.Model):
    refill_hiv = models.CharField(
        verbose_name="Is the patient filling / refilling HIV medications?",
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text=(
            "Select `not applicable` if subject has not "
            "been diagnosed and prescribed medication for HIV infection."
        ),
    )

    class Meta:
        abstract = True


class HtnMedicationsModelMixin(models.Model):
    refill_htn = models.CharField(
        verbose_name="Is the patient filling / refilling Hypertension medications?",
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text=(
            "Select `not applicable` if subject has not "
            "been diagnosed and prescribed medication for Hypertension."
        ),
    )

    class Meta:
        abstract = True


class DmMedicationsModelMixin(models.Model):
    refill_dm = models.CharField(
        verbose_name="Is the patient filling / refilling Diabetes medications?",
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text=(
            "Select `not applicable` if subject has not "
            "been diagnosed and prescribed medication for Diabetes."
        ),
    )

    class Meta:
        abstract = True


class CholMedicationsModelMixin(models.Model):
    refill_chol = models.CharField(
        verbose_name="Is the patient filling / refilling Cholesterol medications?",
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text=(
            "Select `not applicable` if subject has not "
            "been diagnosed and prescribed medication for Cholesterol."
        ),
    )

    class Meta:
        abstract = True



from django.contrib import admin
from edc_model_admin import SimpleHistoryAdmin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from ..admin_site import mocca_prn_admin
from ..models import OnSchedule


@admin.register(OnSchedule, site=mocca_prn_admin)
class OnScheduleAdmin(ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin):
    instructions = None
    fields = ("subject_identifier", "onschedule_datetime")

    list_display = ("subject_identifier", "dashboard", "onschedule_datetime")

    list_filter = ("onschedule_datetime",)

    def get_readonly_fields(self, request, obj=None):
        return ["subject_identifier", "onschedule_datetime"]



# Generated by Django 3.0.9 on 2020-12-05 01:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0017_auto_20201205_0401"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalmoccaregistercontact",
            name="next_appt_date",
            field=models.DateField(null=True, verbose_name="Next Appt."),
        ),
        migrations.AddField(
            model_name="moccaregistercontact",
            name="next_appt_date",
            field=models.DateField(null=True, verbose_name="Next Appt."),
        ),
        migrations.AlterField(
            model_name="historicalmoccaregistercontact",
            name="call_again",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Call again?",
            ),
        ),
        migrations.AlterField(
            model_name="moccaregistercontact",
            name="call_again",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Call again?",
            ),
        ),
    ]



from django.apps import AppConfig as DjangoAppConfig


class AppConfig(DjangoAppConfig):
    name = "mocca_ae"
    verbose_name = "MOCCA: Adverse Events"
    include_in_administration_section = True
    has_exportable_data = True



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_action_item.fieldsets import action_fieldset_tuple
from edc_adverse_event.modeladmin_mixins import DeathReportModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin

from ..admin_site import mocca_ae_admin
from ..forms import DeathReportForm
from ..models import DeathReport


@admin.register(DeathReport, site=mocca_ae_admin)
class DeathReportAdmin(DeathReportModelAdminMixin, SimpleHistoryAdmin):

    form = DeathReportForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_identifier",
                    "report_datetime",
                    "death_datetime",
                    "death_as_inpatient",
                )
            },
        ),
        (
            "Opinion of Local Study Doctor",
            {"fields": ("cause_of_death", "cause_of_death_other", "narrative")},
        ),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )



from .ae import AeListboardView, DeathReportListboardView
from .screening import ListboardView as ScreeningListboardView
from .subject import DashboardView as SubjectDashboardView
from .subject import ListboardView as SubjectListboardView



from django.contrib import admin
from django_audit_fields import audit_fieldset_tuple
from edc_model_admin import (
    ModelAdminFormInstructionsMixin,
    ModelAdminNextUrlRedirectMixin,
    SimpleHistoryAdmin,
    TemplatesModelAdminMixin,
)

from ..admin_site import mocca_screening_admin
from ..forms import CareStatusForm
from ..models import CareStatus
from .fieldsets import care_status_fieldset


@admin.register(CareStatus, site=mocca_screening_admin)
class CareStatusAdmin(
    TemplatesModelAdminMixin,
    ModelAdminFormInstructionsMixin,
    ModelAdminNextUrlRedirectMixin,
    SimpleHistoryAdmin,
):
    form = CareStatusForm

    fieldsets = (
        [None, {"fields": ("mocca_register", "report_datetime", "willing_to_answer")}],
        care_status_fieldset,
        audit_fieldset_tuple,
    )

    list_display = (
        "mocca_register",
        "report_datetime",
        "care",
        "care_facility_location",
        "icc",
        "user_created",
        "created",
    )

    radio_fields = {
        "care": admin.VERTICAL,
        "care_facility_location": admin.VERTICAL,
        "icc": admin.VERTICAL,
        "icc_not_in_reason": admin.VERTICAL,
        "icc_since_mocca": admin.VERTICAL,
    }

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "mocca_register" and request.GET.get("mocca_register"):
            kwargs["queryset"] = db_field.related_model.objects.filter(
                pk=request.GET.get("mocca_register", 0)
            )
        return super().formfield_for_foreignkey(db_field, request, **kwargs)



from django.contrib import admin
from django_audit_fields import audit_fieldset_tuple
from edc_adherence.model_admin_mixin import (
    MedicationAdherenceAdminMixin,
    get_visual_score_fieldset_tuple,
    missed_medications_fieldset_tuple,
)
from edc_form_label.form_label_modeladmin_mixin import FormLabelModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin

from ..admin_site import mocca_subject_admin
from ..forms import DmMedicationAdherenceForm
from ..models import DmMedicationAdherence
from .modeladmin_mixins import CrfModelAdminMixin


@admin.register(DmMedicationAdherence, site=mocca_subject_admin)
class DmMedicationAdherenceAdmin(
    MedicationAdherenceAdminMixin,
    CrfModelAdminMixin,
    FormLabelModelAdminMixin,
    SimpleHistoryAdmin,
):

    form = DmMedicationAdherenceForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        get_visual_score_fieldset_tuple(),
        missed_medications_fieldset_tuple,
        audit_fieldset_tuple,
    )



from django import forms
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_dx_review.utils import raise_if_clinical_review_does_not_exist
from edc_form_validators.form_validator import FormValidator

from ..models import FamilyHistory


class FamilyHistoryFormValidator(CrfFormValidatorMixin, FormValidator):
    def clean(self):
        raise_if_clinical_review_does_not_exist(self.cleaned_data.get("subject_visit"))


class FamilyHistoryForm(CrfModelFormMixin, forms.ModelForm):

    form_validator_cls = FamilyHistoryFormValidator

    class Meta:
        model = FamilyHistory
        fields = "__all__"



from edc_adherence.model_mixins import MedicationAdherenceModelMixin
from edc_model import models as edc_models

from ..model_mixins import CrfModelMixin


class CholMedicationAdherence(
    MedicationAdherenceModelMixin,
    CrfModelMixin,
    edc_models.BaseUuidModel,
):
    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Cholesterol Medication Adherence"
        verbose_name_plural = "Cholesterol Medication Adherence"



from dateutil.relativedelta import relativedelta
from django.contrib.sites.models import Site
from edc_constants.constants import MOBILE_NUMBER, NO, NOT_APPLICABLE, YES
from edc_lab.constants import LT
from edc_utils import get_utcnow
from edc_visit_tracking.constants import SCHEDULED
from faker import Faker
from model_bakery.recipe import Recipe, seq

from mocca_consent.models import SubjectConsent, SubjectReconsent
from mocca_subject.constants import THIS_CLINIC
from mocca_subject.models import (  # TODO: Confirm removal of HealthEconomicsRevised recipe
    ClinicalReview,
    ClinicalReviewBaseline,
    HivInitialReview,
    HtnInitialReview,
)

from .models import SubjectRequisition, SubjectVisit

fake = Faker()


subjectvisit = Recipe(SubjectVisit, reason=SCHEDULED)

subjectrequisition = Recipe(SubjectRequisition)

subjectconsent = Recipe(
    SubjectConsent,
    assessment_score=YES,
    confirm_identity=seq("12315678"),
    consent_copy=YES,
    consent_datetime=get_utcnow(),
    consent_reviewed=YES,
    consent_signature=YES,
    dob=get_utcnow() - relativedelta(years=25),
    first_name=fake.first_name,
    gender="M",
    identity=seq("12315678"),
    identity_type=MOBILE_NUMBER,
    initials="XX",
    is_dob_estimated="-",
    is_incarcerated=NO,
    is_literate=YES,
    last_name=fake.last_name,
    screening_identifier=None,
    study_questions=YES,
    site=Site.objects.get_current(),
    subject_identifier=None,
)

subjectreconsent = Recipe(
    SubjectReconsent,
    site=Site.objects.get_current(),
    consent_reviewed=YES,
    assessment_score=YES,
    study_questions=YES,
    consent_copy=YES,
    action_identifier=None,
    tracking_identifier=None,
)

clinicalreviewbaseline = Recipe(
    ClinicalReviewBaseline,
    site=Site.objects.get_current(),
    hiv_test=YES,
    hiv_test_ago="5y",
    hiv_dx=YES,
    htn_test=NO,
    htn_test_ago=None,
    hypertension=NOT_APPLICABLE,
    dm_test=NO,
    dm_test_ago=None,
    diabetes=NOT_APPLICABLE,
    health_insurance=YES,
    patient_club=YES,
)

clinicalreview = Recipe(
    ClinicalReview,
    site=Site.objects.get_current(),
    hiv_test=NOT_APPLICABLE,
    hiv_test_date=None,
    hiv_dx=NOT_APPLICABLE,
    htn_test=NO,
    htn_test_date=None,
    htn_dx=NOT_APPLICABLE,
    dm_test=NO,
    dm_test_date=None,
    dm_dx=NOT_APPLICABLE,
    health_insurance=YES,
    patient_club=YES,
)

hivinitialreview = Recipe(
    HivInitialReview,
    site=Site.objects.get_current(),
    dx_ago="5y",
    receives_care=YES,
    clinic=THIS_CLINIC,
    arv_initiation_ago="4y",
    has_vl=YES,
    vl=50,
    vl_quantifier=LT,
    has_cd4=NO,
)

htninitialreview = Recipe(
    HtnInitialReview,
    site=Site.objects.get_current(),
    dx_ago="1y",
    receives_care=YES,
    clinic=THIS_CLINIC,
)



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple
from edc_form_label.form_label_modeladmin_mixin import FormLabelModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin

from ..admin_site import mocca_subject_admin
from ..forms import IndicatorsForm
from ..models import Indicators
from .modeladmin_mixins import CrfModelAdminMixin


@admin.register(Indicators, site=mocca_subject_admin)
class IndicatorsAdmin(CrfModelAdminMixin, FormLabelModelAdminMixin, SimpleHistoryAdmin):
    form = IndicatorsForm

    fieldsets = (
        (
            None,
            {
                "description": "Provide if available",
                "fields": ("subject_visit", "report_datetime"),
            },
        ),
        ("Weight and Height", {"fields": ("weight", "height")}),
        (
            "Blood Pressure: Reading 1",
            {
                "description": "Provide if available",
                "fields": (
                    "r1_taken",
                    "r1_reason_not_taken",
                    "sys_blood_pressure_r1",
                    "dia_blood_pressure_r1",
                ),
            },
        ),
        (
            "Blood Pressure: Reading 2",
            {
                "fields": (
                    "r2_taken",
                    "r2_reason_not_taken",
                    "sys_blood_pressure_r2",
                    "dia_blood_pressure_r2",
                )
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "r1_taken": admin.VERTICAL,
        "r2_taken": admin.VERTICAL,
        "crf_status": admin.VERTICAL,
    }

    readonly_fields = []



from django import forms
from django.conf import settings
from edc_constants.constants import NOT_APPLICABLE
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_dx import Diagnoses
from edc_dx.diagnoses import InitialReviewRequired
from edc_form_validators.form_validator import FormValidator
from edc_model import model_exists_or_raise
from edc_utils import convert_php_dateformat
from edc_visit_schedule.utils import is_baseline

from ..models import ClinicalReview, ClinicalReviewBaseline, Medications


class MedicationsFormValidator(CrfFormValidatorMixin, FormValidator):
    def clean(self):
        if not is_baseline(self.cleaned_data.get("subject_visit")):
            model_exists_or_raise(
                self.cleaned_data.get("subject_visit"), model_cls=ClinicalReview
            )
        else:
            model_exists_or_raise(
                self.cleaned_data.get("subject_visit"),
                model_cls=ClinicalReviewBaseline,
                singleton=True,
            )
        self.validate_diagnosis_before_refill()

    def validate_diagnosis_before_refill(self):
        """Assert subject has been diagnosed for the condition
        for which they require a medication refill,
        including for the current timepoint."""

        diagnoses = Diagnoses(
            subject_identifier=self.subject_identifier,
            report_datetime=self.report_datetime,
            lte=True,
        )

        try:
            diagnoses.get_initial_reviews()
        except InitialReviewRequired as e:
            raise forms.ValidationError(e)

        options = []
        for prefix, label in settings.EDC_DX_LABELS.items():
            options.append(
                (
                    f"refill_{prefix}",
                    diagnoses.get_dx(prefix),
                    diagnoses.get_dx_date(prefix),
                    label,
                ),
            )

        for fld, dx, dx_date, label in options:
            if self.cleaned_data.get(fld) == NOT_APPLICABLE and dx:
                formatted_date = dx_date.strftime(convert_php_dateformat(settings.DATE_FORMAT))
                raise forms.ValidationError(
                    {
                        fld: (
                            f"Invalid. Subject was previously diagnosed with {label} "
                            f"on {formatted_date}. Expected YES/NO."
                        )
                    }
                )
            elif self.cleaned_data.get(fld) != NOT_APPLICABLE and not dx:
                raise forms.ValidationError(
                    {
                        fld: (
                            "Invalid. Subject has not been diagnosed with "
                            f"{label}. Expected N/A. See also the "
                            f"`{ClinicalReview._meta.verbose_name}` CRF."
                        )
                    }
                )


class MedicationsForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = MedicationsFormValidator

    class Meta:
        model = Medications
        fields = "__all__"



from typing import List

from django.test import TestCase
from edc_appointment.constants import INCOMPLETE_APPT
from edc_constants.constants import NO, NOT_APPLICABLE, POS, YES
from edc_metadata import KEYED, REQUIRED
from edc_metadata.models import CrfMetadata
from edc_utils import get_utcnow
from edc_visit_tracking.constants import UNSCHEDULED
from model_bakery import baker

from ..mocca_test_case_mixin import MoccaTestCaseMixin


class TestMetadataRules(MoccaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.subject_screening = self.get_subject_screening(report_datetime=get_utcnow())
        self.subject_consent = self.get_subject_consent(
            subject_screening=self.subject_screening
        )

    @staticmethod
    def get_metadata_models(subject_visit) -> List[str]:
        crf_metadatas = CrfMetadata.objects.filter(
            subject_identifier=subject_visit.subject_identifier,
            visit_code=subject_visit.visit_code,
            visit_code_sequence=subject_visit.visit_code_sequence,
        )
        return [
            obj.model
            for obj in crf_metadatas.filter(entry_status__in=[KEYED, REQUIRED]).order_by(
                "model"
            )
        ]

    def test_diagnoses_dates2(self):
        subject_visit_baseline = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
        )

        baker.make(
            "mocca_subject.clinicalreviewbaseline",
            subject_visit=subject_visit_baseline,
            hiv_test=POS,
            hiv_dx=YES,
            hiv_test_ago="5y",
        )

        baker.make(
            "mocca_subject.hivinitialreview",
            subject_visit=subject_visit_baseline,
            dx_ago="5y",
            arv_initiation_ago="4y",
        )
        subject_visit_baseline.appointment.appt_status = INCOMPLETE_APPT
        subject_visit_baseline.appointment.save()
        subject_visit_baseline.appointment.refresh_from_db()
        subject_visit_baseline.refresh_from_db()

        subject_visit = self.get_next_subject_visit(
            subject_visit=subject_visit_baseline, reason=UNSCHEDULED
        )

        clinical_review = baker.make(
            "mocca_subject.clinicalreview",
            subject_visit=subject_visit,
            hiv_test=NOT_APPLICABLE,
            hiv_dx=NOT_APPLICABLE,
            hiv_test_date=None,
            htn_test=NO,
            htn_dx=NOT_APPLICABLE,
            htn_test_date=None,
            dm_test=NO,
            dm_dx=NOT_APPLICABLE,
            dm_test_date=None,
            chol_test=NO,
            chol_dx=NOT_APPLICABLE,
            chol_test_date=None,
        )

        models = self.get_metadata_models(subject_visit)
        self.assertIn("mocca_subject.hivreview", models)
        self.assertNotIn("mocca_subject.hivinitialreview", models)
        self.assertNotIn("mocca_subject.htninitialreview", models)
        self.assertNotIn("mocca_subject.dminitialreview", models)
        self.assertNotIn("mocca_subject.cholinitialreview", models)
        self.assertNotIn("mocca_subject.htnreview", models)
        self.assertNotIn("mocca_subject.dmreview", models)
        self.assertNotIn("mocca_subject.cholreview", models)

        clinical_review.htn_test = YES
        clinical_review.htn_test_date = subject_visit.report_datetime
        clinical_review.htn_dx = YES
        clinical_review.save()
        clinical_review.refresh_from_db()
        self.assertEqual(NOT_APPLICABLE, clinical_review.hiv_test)

        models = self.get_metadata_models(subject_visit)
        self.assertIn("mocca_subject.hivreview", models)
        self.assertIn("mocca_subject.htninitialreview", models)
        self.assertNotIn("mocca_subject.hivinitialreview", models)
        self.assertNotIn("mocca_subject.dminitialreview", models)
        self.assertNotIn("mocca_subject.htnreview", models)
        self.assertNotIn("mocca_subject.dmreview", models)



# Generated by Django 3.0.9 on 2020-12-03 23:27

import _socket
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_utils.date
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0005_auto_20201204_0217"),
    ]

    operations = [
        migrations.CreateModel(
            name="MoccaRegisterContact",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(default=edc_utils.date.get_utcnow),
                ),
                (
                    "contacted",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        default="No",
                        max_length=15,
                    ),
                ),
                (
                    "survival_status",
                    models.CharField(
                        choices=[
                            ("alive", "Alive"),
                            ("dead", "Deceased"),
                            ("unknown", "Unknown"),
                        ],
                        max_length=15,
                        null=True,
                    ),
                ),
                (
                    "willing_to_attend",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")], max_length=15, null=True
                    ),
                ),
                (
                    "call_again",
                    models.CharField(choices=[("Yes", "Yes"), ("No", "No")], max_length=15),
                ),
                (
                    "mocca_register",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="mocca_screening.MoccaRegister",
                    ),
                ),
            ],
            options={
                "verbose_name": "MOCCA Patient Register Contact",
                "verbose_name_plural": "MOCCA Patient Register Contacts",
            },
        ),
    ]



from django.test import TestCase
from edc_appointment.constants import INCOMPLETE_APPT
from edc_constants.constants import CHOL, DM, HIV, HTN, NOT_APPLICABLE, POS, YES
from edc_dx.diagnoses import (
    ClinicalReviewBaselineRequired,
    Diagnoses,
    InitialReviewRequired,
    MultipleInitialReviewsExist,
)
from edc_utils import get_utcnow
from edc_visit_tracking.constants import UNSCHEDULED
from model_bakery import baker

from mocca_screening.constants import HIV_CLINIC

from ..mocca_test_case_mixin import MoccaTestCaseMixin


class TestDiagnoses(MoccaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.subject_screening = self.get_subject_screening(
            report_datetime=get_utcnow(), clinic_type=HIV_CLINIC
        )
        self.subject_consent = self.get_subject_consent(
            subject_screening=self.subject_screening, clinic_type=HIV_CLINIC
        )

    def test_diagnoses(self):
        subject_visit_baseline = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
        )

        subject_visit_baseline.save()

        self.assertRaises(
            ClinicalReviewBaselineRequired,
            Diagnoses,
            subject_identifier=subject_visit_baseline.subject_identifier,
        )

        clinical_review_baseline = baker.make(
            "mocca_subject.clinicalreviewbaseline",
            subject_visit=subject_visit_baseline,
            hiv_test=YES,
            hiv_dx=YES,
            hiv_test_ago="5y",
        )
        try:
            diagnoses = Diagnoses(
                subject_identifier=subject_visit_baseline.subject_identifier,
            )
        except ClinicalReviewBaselineRequired:
            self.fail("DiagnosesError unexpectedly raised")

        self.assertEqual(YES, diagnoses.get_dx(HIV))
        self.assertIsNone(diagnoses.get_dx(HTN))
        self.assertIsNone(diagnoses.get_dx(DM))

        clinical_review_baseline.htn_test = YES
        clinical_review_baseline.htn_test_ago = "1y"
        clinical_review_baseline.htn_dx = YES
        clinical_review_baseline.save()

        diagnoses = Diagnoses(
            subject_identifier=subject_visit_baseline.subject_identifier,
        )
        self.assertEqual(YES, diagnoses.get_dx(HIV))
        self.assertEqual(YES, diagnoses.get_dx(HTN))
        self.assertIsNone(diagnoses.get_dx(DM))

        clinical_review_baseline.dm_test = YES
        clinical_review_baseline.dm_test_ago = "1y"
        clinical_review_baseline.dm_dx = YES
        clinical_review_baseline.save()

        diagnoses = Diagnoses(
            subject_identifier=subject_visit_baseline.subject_identifier,
        )
        self.assertEqual(YES, diagnoses.get_dx(HIV))
        self.assertEqual(YES, diagnoses.get_dx(HTN))
        self.assertEqual(YES, diagnoses.get_dx(DM))

    def test_diagnoses_does_not_raise_for_subject_visit(self):
        """ "Note: Source of the exception will be in
        the metadata rule
        """
        try:
            subject_visit_baseline = self.get_subject_visit(
                subject_screening=self.subject_screening,
                subject_consent=self.subject_consent,
            )
        except ClinicalReviewBaselineRequired:
            self.fail("DiagnosesError unexpectedly raised")

        try:
            subject_visit_baseline.save()
        except ClinicalReviewBaselineRequired:
            self.fail("DiagnosesError unexpectedly raised")

    def test_diagnoses_dates_baseline_raises(self):
        subject_visit_baseline = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
        )

        baker.make(
            "mocca_subject.clinicalreviewbaseline",
            subject_visit=subject_visit_baseline,
            hiv_test=POS,
            hiv_dx=YES,
            hiv_test_ago="5y",
        )

        diagnoses = Diagnoses(
            subject_identifier=subject_visit_baseline.subject_identifier,
        )
        self.assertRaises(InitialReviewRequired, getattr(diagnoses, "get_dx_date"), "hiv")

    def test_diagnoses_dates_baseline(self):
        subject_visit_baseline = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
        )

        clinical_review_baseline = baker.make(
            "mocca_subject.clinicalreviewbaseline",
            subject_visit=subject_visit_baseline,
            hiv_test=POS,
            hiv_dx=YES,
            hiv_test_ago="5y",
        )
        baker.make(
            "mocca_subject.hivinitialreview",
            subject_visit=subject_visit_baseline,
            dx_ago="5y",
            arv_initiation_ago="4y",
        )
        diagnoses = Diagnoses(
            subject_identifier=subject_visit_baseline.subject_identifier,
        )

        self.assertEqual(YES, diagnoses.get_dx(HIV))
        self.assertIsNone(diagnoses.get_dx(DM))
        self.assertIsNone(diagnoses.get_dx(HTN))
        self.assertIsNone(diagnoses.get_dx(CHOL))
        self.assertEqual(
            diagnoses.get_dx_date("hiv"),
            clinical_review_baseline.hiv_test_estimated_date,
        )
        self.assertIsNone(diagnoses.get_dx_date(DM))
        self.assertIsNone(diagnoses.get_dx_date(HTN))
        self.assertIsNone(diagnoses.get_dx_date(CHOL))

        diagnoses = Diagnoses(
            subject_identifier=subject_visit_baseline.subject_identifier,
            report_datetime=subject_visit_baseline.report_datetime,
            lte=True,
        )

        self.assertEqual(YES, diagnoses.get_dx(HIV))
        self.assertIsNone(diagnoses.get_dx(DM))
        self.assertIsNone(diagnoses.get_dx(HTN))
        self.assertIsNone(diagnoses.get_dx(CHOL))
        self.assertEqual(
            diagnoses.get_dx_date(HIV),
            clinical_review_baseline.hiv_test_estimated_date,
        )
        self.assertIsNone(diagnoses.get_dx_date(DM))
        self.assertIsNone(diagnoses.get_dx_date(HTN))
        self.assertIsNone(diagnoses.get_dx_date(CHOL))

        diagnoses = Diagnoses(
            subject_identifier=subject_visit_baseline.subject_identifier,
            report_datetime=subject_visit_baseline.report_datetime,
            lte=False,
        )

        self.assertEqual(YES, diagnoses.get_dx(HIV))
        self.assertIsNone(diagnoses.get_dx(DM))
        self.assertIsNone(diagnoses.get_dx(HTN))
        self.assertIsNone(diagnoses.get_dx(CHOL))
        self.assertEqual(
            diagnoses.get_dx_date(HIV),
            clinical_review_baseline.hiv_test_estimated_date,
        )
        self.assertIsNone(diagnoses.get_dx_date(DM))
        self.assertIsNone(diagnoses.get_dx_date(HTN))
        self.assertIsNone(diagnoses.get_dx_date(CHOL))

    def test_diagnoses_dates(self):
        subject_visit_baseline = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
        )

        baker.make(
            "mocca_subject.clinicalreviewbaseline",
            subject_visit=subject_visit_baseline,
            hiv_test=POS,
            hiv_dx=YES,
            hiv_test_ago="5y",
        )

        hiv_initial_review = baker.make(
            "mocca_subject.hivinitialreview",
            subject_visit=subject_visit_baseline,
            dx_ago="5y",
            arv_initiation_ago="4y",
        )

        subject_visit_baseline.appointment.appt_status = INCOMPLETE_APPT
        subject_visit_baseline.appointment.save()
        subject_visit_baseline.appointment.refresh_from_db()
        subject_visit_baseline.refresh_from_db()

        subject_visit = self.get_next_subject_visit(
            subject_visit=subject_visit_baseline, reason=UNSCHEDULED
        )

        baker.make(
            "mocca_subject.clinicalreview",
            subject_visit=subject_visit,
            hiv_test=NOT_APPLICABLE,
            hiv_dx=NOT_APPLICABLE,
            hiv_test_date=None,
            htn_test=YES,
            htn_dx=YES,
            htn_test_date=subject_visit.report_datetime,
        )

        htn_initial_review = baker.make(
            "mocca_subject.htninitialreview",
            subject_visit=subject_visit,
            dx_ago=None,
            dx_date=subject_visit.report_datetime,
        )

        diagnoses = Diagnoses(
            subject_identifier=subject_visit.subject_identifier,
            report_datetime=subject_visit.report_datetime,
            lte=True,
        )
        self.assertIsNotNone(diagnoses.get_dx_date("hiv"))
        self.assertEqual(
            diagnoses.get_dx_date("hiv"),
            hiv_initial_review.get_best_dx_date().date(),
        )

        self.assertEqual(
            diagnoses.get_dx_date("htn"),
            htn_initial_review.get_best_dx_date().date(),
        )
        self.assertIsNotNone(diagnoses.get_dx_date("htn"))

    def test_diagnoses_dates_baseline2(self):
        subject_visit_baseline = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
        )

        baker.make(
            "mocca_subject.clinicalreviewbaseline",
            subject_visit=subject_visit_baseline,
            hiv_test=POS,
            hiv_dx=YES,
            hiv_test_ago="5y",
        )
        baker.make(
            "mocca_subject.hivinitialreview",
            subject_visit=subject_visit_baseline,
            dx_ago="5y",
            arv_initiation_ago="4y",
        )
        subject_visit_baseline.appointment.appt_status = INCOMPLETE_APPT
        subject_visit_baseline.appointment.save()
        subject_visit_baseline.appointment.refresh_from_db()
        subject_visit_baseline.refresh_from_db()

        subject_visit = self.get_next_subject_visit(
            subject_visit=subject_visit_baseline, reason=UNSCHEDULED
        )

        baker.make(
            "mocca_subject.hivinitialreview",
            subject_visit=subject_visit,
            dx_ago="5y",
            arv_initiation_ago="4y",
        )

        diagnoses = Diagnoses(
            subject_identifier=subject_visit_baseline.subject_identifier,
        )
        self.assertRaises(MultipleInitialReviewsExist, diagnoses.get_initial_reviews)



from django.test import TestCase, tag
from edc_constants.constants import COMPLETE, NO, NONSMOKER, NOT_APPLICABLE, SMOKER
from model_bakery import baker

from mocca_subject.forms import OtherBaselineDataForm

from ..mocca_test_case_mixin import MoccaTestCaseMixin


class TestOtherBaselineData(MoccaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.data = {
            "crf_status": COMPLETE,
            "smoking_status": SMOKER,
            "smoker_quit_ago_str": "1y2m",
            "alcohol": NO,
            "alcohol_consumption": NOT_APPLICABLE,
            "employment_status": "professional",
            "education": "primary",
            "marital_status": "married",
        }

        self.subject_visit = self.get_subject_visit()

        baker.make("mocca_subject.clinicalreviewbaseline", subject_visit=self.subject_visit)

        self.data.update(
            subject_visit=self.subject_visit.pk,
            report_datetime=self.subject_visit.report_datetime,
        )

    @tag("other_baseline")
    def test_ok(self):
        form = OtherBaselineDataForm(data=self.data)
        form.is_valid()
        self.assertEqual(form._errors, {})

    @tag("other_baseline")
    def test_non_smoker(self):
        self.data.update({"smoking_status": NONSMOKER, "smoker_quit_ago_str": None})
        form = OtherBaselineDataForm(data=self.data)
        form.is_valid()
        self.assertEqual(form._errors, {})



from django.contrib.admin import AdminSite as DjangoAdminSite


class AdminSite(DjangoAdminSite):

    site_title = "MOCCA Export"
    site_header = "MOCCA Export"
    index_title = "MOCCA Export"
    site_url = "/administration/"
    enable_nav_sidebar = False  # DJ 3.1


mocca_export_admin = AdminSite(name="mocca_export_admin")



from edc_constants.constants import OTHER, UNKNOWN

DEATH_LOCATIONS = (
    ("home", "At home"),
    ("hospital_clinic", "Hospital/clinic"),
    ("home", "Elsewhere"),
)


INFORMANT_RELATIONSHIP = (
    ("husband_wife", "Husband/wife"),
    ("Parent", "Parent"),
    ("child", "Child"),
    (UNKNOWN, "Unknown"),
    (OTHER, "Other"),
)



from django.db import models
from edc_model.models import BaseUuidModel, OtherCharField
from edc_model.models.historical_records import HistoricalRecords
from edc_search.model_mixins import SearchSlugManager
from edc_sites.models import CurrentSiteManager, SiteModelMixin
from edc_utils import get_utcnow

from ..choices import REFUSAL_REASONS


class SubjectRefusalManager(SearchSlugManager, models.Manager):
    def get_by_natural_key(self, screening_identifier):
        return self.get(screening_identifier=screening_identifier)


class SubjectRefusal(SiteModelMixin, BaseUuidModel):
    screening_identifier = models.CharField(max_length=50, unique=True)

    report_datetime = models.DateTimeField(
        verbose_name="Report Date and Time", default=get_utcnow
    )

    reason = models.CharField(
        verbose_name="Reason for refusal to join",
        max_length=25,
        choices=REFUSAL_REASONS,
    )

    other_reason = OtherCharField()

    comment = models.TextField(
        verbose_name="Additional Comments",
        null=True,
        blank=True,
    )

    on_site = CurrentSiteManager()

    objects = SubjectRefusalManager()

    history = HistoricalRecords()

    def __str__(self):
        return self.screening_identifier

    def natural_key(self):
        return (self.screening_identifier,)

    @staticmethod
    def get_search_slug_fields():
        return ["screening_identifier"]

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Refusal to Consent"
        verbose_name_plural = "Refusal to Consent"



from edc_constants.constants import (
    DIABETES,
    HIV,
    HYPERTENSION,
    NO,
    NOT_APPLICABLE,
    OTHER,
    PURPOSIVELY_SELECTED,
    RANDOM_SAMPLING,
    UNKNOWN,
)

from .constants import (
    DIABETES_CLINIC,
    HIV_CLINIC,
    HYPERTENSION_CLINIC,
    INTEGRATED,
    NCD,
    NCD_CLINIC,
    NO_INTERRUPTION,
    SEQUENTIAL,
    SOME_INTERRUPTION,
    SYSTEMATIC,
)

CLINIC_CHOICES = (
    (HIV_CLINIC, "HIV Clinic"),
    (NCD_CLINIC, "NCD Clinic (Joint Diabetes/Hypertension)"),
    (DIABETES_CLINIC, "Diabetes Clinic"),
    (HYPERTENSION_CLINIC, "Hypertension Clinic"),
    (INTEGRATED, "Integrated Clinic"),
)

REFUSAL_REASONS = (
    ("unwilling_to_say", "Unwilling to give any reason"),
    ("worried_about_care", "Worried that he/she will not get the care needed"),
    (
        "mixing_and_stigme",
        "Does not want to mix with patients with other conditions "
        "because of concern around stigma",
    ),
    (
        "mixing_and_infection",
        "Does not want to mix with patients with other conditions because "
        "of concerns around contracting infections",
    ),
    (
        "happy_vertical_care",
        "Patient is happy with the care being given in vertical clinics",
    ),
    (
        "no_participant",
        "No longer wishes to be a study participant but is happy to continue "
        "receiving care in the integrated clinic",
    ),
    (
        "receives_community_care",
        "Patient is receiving HIV care in the community and does not wish "
        "to return to facility-based care",
    ),
    (
        "unable_to_contact",
        "Clinic was unable to contact the subject",
    ),
    (OTHER, "Other, please specify"),
)
REFUSAL_REASONS_SCREENING = REFUSAL_REASONS
CLINIC_DAYS = (
    (INTEGRATED, "Integrated care day (HIV, Diabetes, Hypertension)"),
    (NCD, "NCD day (Diabetes + Hypertension)"),
    (HIV, "HIV only day"),
    (DIABETES, "Diabetes only day"),
    (HYPERTENSION, "Hypertension only day"),
)

SELECTION_METHOD = (
    (RANDOM_SAMPLING, "Random sampling"),
    (SYSTEMATIC, "Systematically selected"),
    (SEQUENTIAL, "Sequentially selected"),
    (PURPOSIVELY_SELECTED, "Purposively selected"),
)

RESPONDENT_CHOICES = (
    ("patient", "Patient"),
    ("family", "Family"),
    ("friend", "friend"),
    ("other", "other"),
    (NOT_APPLICABLE, "Not applicable"),
)

CARE_SINCE_MOCCA = (
    (NO_INTERRUPTION, "Yes, without interruption"),
    (SOME_INTERRUPTION, "Yes, with some interruption"),
    (NO, "No, not since completing follow up with the MOCCA (original) trial."),
    (UNKNOWN, "Unknown"),
)

NOT_ICC_REASONS = (
    ("icc_not_available", "ICC not available (or closed) in this facility"),
    ("moved", "Moved out of area"),
    ("dont_want", "Personally chose not to continue with integrated care"),
    (
        "advised_to_vertical",
        "Healthcare staff asked patient to return to vertical care",
    ),
    ("referred_out", "Referred to another facility without an ICC"),
    (
        "receives_community_care",
        "Patient is currently receiving their HIV care in the community",
    ),
    (NOT_APPLICABLE, "Not applicable"),
)



from edc_constants.constants import YES
from edc_form_validators.form_validator import FormValidator
from edc_lab.form_validators import CrfRequisitionFormValidatorMixin
from edc_reportable import ReportablesFormValidatorMixin
from edc_reportable.constants import GRADE3, GRADE4


class BloodResultsFormValidatorMixin(
    ReportablesFormValidatorMixin, CrfRequisitionFormValidatorMixin, FormValidator
):

    reportable_grades = [GRADE3, GRADE4]
    reference_list_name = "mocca"
    requisition_field = None
    assay_datetime_field = None
    field_names = []
    panels = []
    poc_panels = []

    @property
    def field_values(self):
        return [self.cleaned_data.get(f) is not None for f in [f for f in self.field_names]]

    @property
    def extra_options(self):
        return {}

    def clean(self):
        self.required_if_true(any(self.field_values), field_required=self.requisition_field)

        if self.cleaned_data.get("is_poc") and self.cleaned_data.get("is_poc") == YES:
            self.validate_requisition(
                self.requisition_field, self.assay_datetime_field, *self.poc_panels
            )
        else:
            self.validate_requisition(
                self.requisition_field, self.assay_datetime_field, *self.panels
            )

        # for field_name in self.field_names:
        #     self.validate_units_field(field_name)
        #     # self.validate_abnormal_field(field_name)
        #     # self.validate_reportable_field(field_name)

        self.validate_reportable_fields(
            reference_range_collection_name=self.reference_list_name, **self.extra_options
        )



from django import forms
from edc_constants.constants import FORMER_SMOKER, YES
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators.form_validator import FormValidator
from edc_model.utils import estimated_date_from_ago
from edc_visit_schedule.utils import raise_if_not_baseline

from ..models import OtherBaselineData


class OtherBaselineDataFormValidator(CrfFormValidatorMixin, FormValidator):
    def clean(self):
        raise_if_not_baseline(self.cleaned_data.get("subject_visit"))

        self.required_if(
            FORMER_SMOKER, field="smoking_status", field_required="smoker_quit_ago"
        )
        estimated_date_from_ago(data=self.cleaned_data, ago_field="smoker_quit_ago")

        self.applicable_if(YES, field="alcohol", field_applicable="alcohol_consumption")

        self.validate_other_specify(
            field="employment_status", other_specify_field="employment_status_other"
        )


class OtherBaselineDataForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = OtherBaselineDataFormValidator

    class Meta:
        model = OtherBaselineData
        fields = "__all__"



from django import forms
from edc_constants.constants import NO, UNKNOWN, YES
from edc_form_validators import FormValidator, FormValidatorMixin

from mocca_screening.models import CareStatus

from ..constants import NO_INTERRUPTION, SOME_INTERRUPTION


class CareStatusFormValidatorMixin(FormValidator):
    def validate_care_options(self) -> None:
        self.required_if(NO, UNKNOWN, field="care", field_required="care_not_in_reason")
        self.applicable_if(YES, field="care", field_applicable="care_facility_location")
        self.applicable_if(YES, field="care", field_applicable="icc")
        self.applicable_if(NO, field="icc", field_applicable="icc_not_in_reason")
        self.required_if(
            SOME_INTERRUPTION,
            field="icc_since_mocca",
            field_required="icc_since_mocca_comment",
        )
        if (
            self.cleaned_data.get("icc_since_mocca") == NO
            and self.cleaned_data.get("icc") == YES
        ):
            raise forms.ValidationError(
                {"icc_since_mocca": "Invalid. Patient is currently receiving integrated care."}
            )
        if (
            self.cleaned_data.get("icc_since_mocca") == NO_INTERRUPTION
            and self.cleaned_data.get("icc") != YES
        ):
            raise forms.ValidationError(
                {
                    "icc_since_mocca": (
                        "Invalid. Patient is NOT currently receiving integrated care."
                    )
                }
            )
        return None


class CareStatusFormValidator(CareStatusFormValidatorMixin):
    pass


class CareStatusForm(FormValidatorMixin, forms.ModelForm):
    form_validator_cls = CareStatusFormValidator

    class Meta:
        model = CareStatus
        fields = "__all__"



from django.apps import AppConfig as DjangoApponfig


class AppConfig(DjangoApponfig):
    name = "mocca_screening"
    verbose_name = "MOCCA: Screening"
    screening_age_adult_upper = 99
    screening_age_adult_lower = 18
    include_in_administration_section = True
    has_exportable_data = True



from django.db import models
from edc_model import models as edc_models

from mocca_lists.models import DmTreatments

from ..model_mixins import CrfModelMixin
from .model_mixins import DrugRefillModelMixin


class DrugRefillDm(DrugRefillModelMixin, CrfModelMixin, edc_models.BaseUuidModel):

    rx = models.ManyToManyField(
        DmTreatments,
        verbose_name="Which medicine did the patient receive today?",
    )

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Drug Refill: Diabetes"
        verbose_name_plural = "Drug Refills: Diabetes"



from django.contrib import admin
from django_audit_fields import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple
from edc_form_label import FormLabelModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin

from ..admin_site import mocca_subject_admin
from ..forms import NextAppointmentForm
from ..models import NextAppointment
from .modeladmin_mixins import CrfModelAdminMixin


@admin.register(NextAppointment, site=mocca_subject_admin)
class NextAppointmentAdmin(CrfModelAdminMixin, FormLabelModelAdminMixin, SimpleHistoryAdmin):
    form = NextAppointmentForm
    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime", "appt_date")}),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "crf_status": admin.VERTICAL,
    }



# Generated by Django 3.0.9 on 2020-12-03 14:57

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0003_auto_20201203_1610"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalmoccaregister",
            name="screening_identifier",
            field=models.CharField(
                max_length=15,
                null=True,
                verbose_name="MOCCA (ext) screening identifier",
            ),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="screening_identifier",
            field=models.CharField(
                max_length=15,
                null=True,
                verbose_name="MOCCA (ext) screening identifier",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="mocca_register",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"screening_identifier__isnull": True},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="mocca_screening.MoccaRegister",
                verbose_name="Select one from the MOCCA (original) register",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="mocca_register",
            field=models.OneToOneField(
                limit_choices_to={"screening_identifier__isnull": True},
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="mocca_screening.MoccaRegister",
                verbose_name="Select one from the MOCCA (original) register",
            ),
        ),
    ]



from django.contrib import admin
from django.template.loader import render_to_string
from django.urls.base import reverse
from django.urls.exceptions import NoReverseMatch
from django.utils.safestring import mark_safe
from django.utils.translation import gettext as _
from django_audit_fields.admin import audit_fieldset_tuple
from edc_dashboard.url_names import url_names
from edc_model_admin import SimpleHistoryAdmin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_screening import format_reasons_ineligible

from ..admin_site import mocca_screening_admin
from ..forms import SubjectScreeningForm
from ..models import SubjectScreening
from .fieldsets import care_status_fieldset


@admin.register(SubjectScreening, site=mocca_screening_admin)
class SubjectScreeningAdmin(ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin):
    form = SubjectScreeningForm

    post_url_on_delete_name = "screening_listboard_url"
    subject_listboard_url_name = "screening_listboard_url"

    additional_instructions = (
        "Patients must meet ALL of the inclusion criteria and NONE of the "
        "exclusion criteria in order to proceed"
    )

    fieldsets = (
        [None, {"fields": ("report_datetime",)}],
        [
            "Original MOCCA information",
            {
                "fields": (
                    "mocca_participant",
                    "mocca_register",
                    "screening_consent",
                )
            },
        ],
        care_status_fieldset,
        [
            "Consent",
            {
                "fields": (
                    "willing_to_consent",
                    "pregnant",
                    "requires_acute_care",
                )
            },
        ],
        audit_fieldset_tuple,
    )

    list_display = (
        "screening_identifier",
        "eligiblity_status",
        "demographics",
        "integrated_care",
        "will_consent",
        "reasons",
        "report_datetime",
        "user_created",
        "created",
    )

    list_filter = (
        "report_datetime",
        "gender",
        "eligible",
        "consented",
        "refused",
        "eligible",
        "icc",
        "willing_to_consent",
    )

    search_fields = (
        "screening_identifier",
        "subject_identifier",
        "mocca_register__mocca_study_identifier",
        "initials",
        "reasons_ineligible",
    )

    radio_fields = {
        "mocca_participant": admin.VERTICAL,
        "mocca_site": admin.VERTICAL,
        "gender": admin.VERTICAL,
        "screening_consent": admin.VERTICAL,
        "unsuitable_agreed": admin.VERTICAL,
        "care": admin.VERTICAL,
        "care_facility_location": admin.VERTICAL,
        "icc": admin.VERTICAL,
        "icc_not_in_reason": admin.VERTICAL,
        "icc_since_mocca": admin.VERTICAL,
        "willing_to_consent": admin.VERTICAL,
        "pregnant": admin.VERTICAL,
        "requires_acute_care": admin.VERTICAL,
    }

    def get_readonly_fields(self, request, obj=None):
        fields = super().get_readonly_fields(request, obj=obj)
        if obj and "mocca_register" not in fields:
            fields = list(fields)
            fields.append("mocca_register")
            fields = tuple(fields)
        return fields

    def post_url_on_delete_kwargs(self, request, obj):
        return {}

    def demographics(self, obj=None):
        return mark_safe(
            f"{obj.get_gender_display()} {obj.age_in_years}yrs {obj.initials.upper()}"
        )

    def integrated_care(self, obj=None):
        return obj.get_icc_display()

    def will_consent(self, obj=None):
        return obj.get_willing_to_consent_display()

    def reasons(self, obj=None):
        return format_reasons_ineligible(obj.reasons_ineligible)

    def eligiblity_status(self, obj=None):
        return "Eligible" if obj.eligible else "Ineligible"

    def dashboard(self, obj=None, label=None):
        try:
            url = reverse(
                self.get_subject_dashboard_url_name(),
                kwargs=self.get_subject_dashboard_url_kwargs(obj),
            )
        except NoReverseMatch:
            url = reverse(url_names.get("screening_listboard_url"), kwargs={})
            context = dict(
                title=_("Go to screening listboard"),
                url=f"{url}?q={obj.screening_identifier}",
                label=label,
            )
        else:
            context = dict(title=_("Go to subject dashboard"), url=url, label=label)
        return render_to_string("dashboard_button.html", context=context)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "mocca_register" and request.GET.get("mocca_register"):
            kwargs["queryset"] = db_field.related_model.objects.filter(
                pk=request.GET.get("mocca_register", 0)
            )
        return super().formfield_for_foreignkey(db_field, request, **kwargs)



# Generated by Django 3.0.9 on 2020-12-16 01:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0008_auto_20201216_0417"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalmoccaregister",
            name="screen_now",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                max_length=15,
                verbose_name="Patient is present. Screen now?",
            ),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="screen_now",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                max_length=15,
                verbose_name="Patient is present. Screen now?",
            ),
        ),
    ]



import pytz
from arrow.arrow import Arrow
from django import forms
from django.apps import apps as django_apps
from django.conf import settings
from edc_form_validators.base_form_validator import INVALID_ERROR
from edc_utils import convert_php_dateformat


class StudyDayFormValidatorMixin:
    def validate_study_day_with_datetime(
        self,
        subject_identifier=None,
        study_day=None,
        compare_date=None,
        study_day_field=None,
    ):
        """Raises an exception if study day does not match
        calculation against pytz.

        Note: study-day is 1-based.
        """
        if study_day is not None and compare_date is not None:
            try:
                compare_date = compare_date.date()
            except AttributeError:
                pass
            subject_identifier = (
                subject_identifier
                or self.cleaned_data.get("subject_identifier")
                or self.instance.subject_identifier
            )
            if not subject_identifier:
                raise ValueError(f"Subject identifier cannot be None. See {repr(self)}")
            registered_subject_model_cls = django_apps.get_model(
                "edc_registration.registeredsubject"
            )
            randomization_datetime = registered_subject_model_cls.objects.get(
                subject_identifier=subject_identifier
            ).randomization_datetime
            days_on_study = (compare_date - randomization_datetime.date()).days
            if study_day - 1 != days_on_study:
                tz = pytz.timezone(settings.TIME_ZONE)
                formatted_date = (
                    Arrow.fromdatetime(randomization_datetime)
                    .to(tz)
                    .strftime(convert_php_dateformat(settings.DATETIME_FORMAT))
                )
                message = {
                    study_day_field: (
                        f"Invalid. Expected {days_on_study + 1}. "
                        f"Subject was registered on {formatted_date}"
                    )
                }
                print(message)
                self._errors.update(message)
                self._error_codes.append(INVALID_ERROR)
                raise forms.ValidationError(message, code=INVALID_ERROR)



from django.contrib import admin
from django.utils.safestring import mark_safe
from django_audit_fields.admin import audit_fieldset_tuple
from edc_model_admin import ModelAdminFormInstructionsMixin, TemplatesModelAdminMixin
from edc_model_admin.model_admin_simple_history import SimpleHistoryAdmin

from ..admin_site import mocca_screening_admin
from ..forms import DailyClosingLogForm
from ..models import DailyClosingLog


@admin.register(DailyClosingLog, site=mocca_screening_admin)
class DailyClosingLogAdmin(
    TemplatesModelAdminMixin, ModelAdminFormInstructionsMixin, SimpleHistoryAdmin
):
    form = DailyClosingLogForm
    date_hierarchy = "log_date"
    show_object_tools = True
    additional_instructions = mark_safe(
        "<font color='orange'><B>This form should be completed "
        "at the end of each clinic day.</B></font>"
    )

    fieldsets = (
        [None, {"fields": ("log_date", "site")}],
        [
            "Daily Closing Log",
            {
                "fields": (
                    "clinic_services",
                    "attended",
                    "selection_method",
                    "approached",
                    "agreed_to_screen",
                    "comment",
                )
            },
        ],
        audit_fieldset_tuple,
    )

    list_display = (
        "log_date",
        "clinic_services",
        "number_attended",
        "number_approached",
        "number_agreed_to_screen",
    )

    list_filter = ("log_date", "clinic_services", "created", "modified")

    radio_fields = {
        "clinic_services": admin.VERTICAL,
        "selection_method": admin.VERTICAL,
    }

    def number_attended(self, obj):
        return obj.attended

    number_attended.short_description = "Attended"

    def number_approached(self, obj):
        return obj.approached

    number_approached.short_description = "Approached"

    def number_agreed_to_screen(self, obj):
        return obj.agreed_to_screen

    number_agreed_to_screen.short_description = "Agreed to Screen"

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "site":
            try:
                site_id = request.site.id
            except AttributeError:
                site_id = None
            kwargs["queryset"] = db_field.related_model.objects.filter(pk=site_id)
        return super().formfield_for_foreignkey(db_field, request, **kwargs)



# Generated by Django 3.1.6 on 2021-02-28 17:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_lists', '0003_auto_20210225_0728'),
    ]

    operations = [
        migrations.CreateModel(
            name='DiagnosisLocations',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, help_text='This is the stored value, required', max_length=250, unique=True, verbose_name='Stored value')),
                ('display_name', models.CharField(db_index=True, help_text='(suggest 40 characters max.)', max_length=250, unique=True, verbose_name='Name')),
                ('display_index', models.IntegerField(db_index=True, default=0, help_text='Index to control display order if not alphabetical, not required', verbose_name='display index')),
                ('field_name', models.CharField(blank=True, editable=False, help_text='Not required', max_length=25, null=True)),
                ('version', models.CharField(default='1.0', editable=False, max_length=35)),
            ],
            options={
                'verbose_name': 'Diagnosis Locations',
                'verbose_name_plural': 'Diagnosis Locations',
                'ordering': ['display_index', 'display_name'],
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'export', 'import'),
            },
        ),
        migrations.AddIndex(
            model_name='diagnosislocations',
            index=models.Index(fields=['id', 'display_name', 'display_index'], name='mocca_lists_id_a18628_idx'),
        ),
    ]



from django.conf import settings
from django.db import models
from edc_model import models as edc_models


class DiagnosisLocationModelMixin(models.Model):

    dx_location = models.ForeignKey(
        f"{settings.LIST_MODEL_APP_LABEL}.diagnosislocations",
        verbose_name="Where was the diagnosis made?",
        on_delete=models.PROTECT,
    )

    dx_location_other = edc_models.OtherCharField()

    class Meta:
        abstract = True



# Generated by Django 3.1.6 on 2021-03-02 04:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_subject', '0016_auto_20210302_0736'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='drugrefillhiv',
            name='clinic_days',
        ),
        migrations.RemoveField(
            model_name='drugrefillhiv',
            name='club_days',
        ),
        migrations.RemoveField(
            model_name='drugrefillhiv',
            name='purchased_days',
        ),
        migrations.RemoveField(
            model_name='historicaldrugrefillhiv',
            name='clinic_days',
        ),
        migrations.RemoveField(
            model_name='historicaldrugrefillhiv',
            name='club_days',
        ),
        migrations.RemoveField(
            model_name='historicaldrugrefillhiv',
            name='purchased_days',
        ),
    ]



# Generated by Django 3.0.9 on 2020-12-03 13:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0002_auto_20201203_0338"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalmoccaregister",
            name="gender",
            field=models.CharField(
                choices=[("M", "Male"), ("F", "Female")], max_length=10, null=True
            ),
        ),
        migrations.AlterField(
            model_name="moccaregister",
            name="gender",
            field=models.CharField(
                choices=[("M", "Male"), ("F", "Female")], max_length=10, null=True
            ),
        ),
    ]



# Generated by Django 3.0.9 on 2021-01-15 01:23

import edc_model.models.fields.other_charfield
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_subject", "0006_auto_20210115_0411"),
    ]

    operations = [
        migrations.AddField(
            model_name="dminitialreview",
            name="dx_location_other",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicaldminitialreview",
            name="dx_location_other",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalhivinitialreview",
            name="dx_location_other",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalhtninitialreview",
            name="dx_location_other",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="hivinitialreview",
            name="dx_location_other",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="htninitialreview",
            name="dx_location_other",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="dminitialreview",
            name="dx_location",
            field=models.CharField(
                choices=[
                    ("hospital", "Hospital"),
                    ("gov_clinic", "Government clinic"),
                    ("private_clinic", "Private clinic"),
                    ("private_doctor", "Private doctor"),
                    ("mocca_clinic", "MOCCA study clinic"),
                    ("unknown", "Don't recall"),
                    ("OTHER", "Other, specify"),
                ],
                max_length=25,
                verbose_name="Where was the diagnosis made?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldminitialreview",
            name="dx_location",
            field=models.CharField(
                choices=[
                    ("hospital", "Hospital"),
                    ("gov_clinic", "Government clinic"),
                    ("private_clinic", "Private clinic"),
                    ("private_doctor", "Private doctor"),
                    ("mocca_clinic", "MOCCA study clinic"),
                    ("unknown", "Don't recall"),
                    ("OTHER", "Other, specify"),
                ],
                max_length=25,
                verbose_name="Where was the diagnosis made?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhivinitialreview",
            name="dx_location",
            field=models.CharField(
                choices=[
                    ("hospital", "Hospital"),
                    ("gov_clinic", "Government clinic"),
                    ("private_clinic", "Private clinic"),
                    ("private_doctor", "Private doctor"),
                    ("mocca_clinic", "MOCCA study clinic"),
                    ("unknown", "Don't recall"),
                    ("OTHER", "Other, specify"),
                ],
                max_length=25,
                verbose_name="Where was the diagnosis made?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhtninitialreview",
            name="dx_location",
            field=models.CharField(
                choices=[
                    ("hospital", "Hospital"),
                    ("gov_clinic", "Government clinic"),
                    ("private_clinic", "Private clinic"),
                    ("private_doctor", "Private doctor"),
                    ("mocca_clinic", "MOCCA study clinic"),
                    ("unknown", "Don't recall"),
                    ("OTHER", "Other, specify"),
                ],
                max_length=25,
                verbose_name="Where was the diagnosis made?",
            ),
        ),
        migrations.AlterField(
            model_name="hivinitialreview",
            name="dx_location",
            field=models.CharField(
                choices=[
                    ("hospital", "Hospital"),
                    ("gov_clinic", "Government clinic"),
                    ("private_clinic", "Private clinic"),
                    ("private_doctor", "Private doctor"),
                    ("mocca_clinic", "MOCCA study clinic"),
                    ("unknown", "Don't recall"),
                    ("OTHER", "Other, specify"),
                ],
                max_length=25,
                verbose_name="Where was the diagnosis made?",
            ),
        ),
        migrations.AlterField(
            model_name="htninitialreview",
            name="dx_location",
            field=models.CharField(
                choices=[
                    ("hospital", "Hospital"),
                    ("gov_clinic", "Government clinic"),
                    ("private_clinic", "Private clinic"),
                    ("private_doctor", "Private doctor"),
                    ("mocca_clinic", "MOCCA study clinic"),
                    ("unknown", "Don't recall"),
                    ("OTHER", "Other, specify"),
                ],
                max_length=25,
                verbose_name="Where was the diagnosis made?",
            ),
        ),
    ]



from edc_crf.crf_model_mixin import CrfModelMixin as BaseCrfModelMixin
from edc_crf.crf_status_model_mixin import CrfStatusModelMixin


class CrfModelMixin(CrfStatusModelMixin, BaseCrfModelMixin):
    class Meta(BaseCrfModelMixin.Meta):
        abstract = True



from .end_of_study_admin import EndOfStudyAdmin
from .loss_to_followup_admin import LossToFollowupAdmin
from .offschedule_admin import OffScheduleAdmin
from .onschedule_admin import OnScheduleAdmin
from .protocol_deviation_violation_admin import ProtocolDeviationViolationAdmin



from django import forms
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators.form_validator import FormValidator

from ..models import DrugRefillDm
from .drug_refill_mixins import DrugRefillFormValidatorMixin


class DrugRefillDmFormValidator(
    DrugRefillFormValidatorMixin, CrfFormValidatorMixin, FormValidator
):
    def clean(self):
        self.drug_refill_clean()


class DrugRefillDmForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = DrugRefillDmFormValidator

    class Meta:
        model = DrugRefillDm
        fields = "__all__"



from django import forms
from edc_adverse_event.forms import DeathReportTmgModelFormMixin

from ..models import DeathReportTmg


class DeathReportTmgForm(DeathReportTmgModelFormMixin, forms.ModelForm):
    class Meta:
        model = DeathReportTmg
        fields = "__all__"



# from .blood_results_form_validator import (
#     BloodResultsGluFormValidator,
#     BloodResultsHba1cFormValidator,
#     BloodResultsLipidFormValidator,
# )
# from .blood_results_form_validator_mixin import BloodResultsFormValidatorMixin



from django import forms
from edc_constants.constants import YES
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators import FormValidator
from edc_glucose.utils import validate_glucose_as_millimoles_per_liter

from ..models import GlucoseFollowup


class GlucoseFollowupFormValidator(FormValidator):
    required_at_baseline = False

    def clean(self) -> None:
        self.applicable_if(YES, field="glucose_performed", field_applicable="glucose_fasted")
        self.required_if(YES, field="glucose_performed", field_required="glucose_date")
        self.required_if(YES, field="glucose_performed", field_required="glucose_value")
        self.required_if(YES, field="glucose_performed", field_required="glucose_quantifier")
        self.required_if(YES, field="glucose_performed", field_required="glucose_units")
        validate_glucose_as_millimoles_per_liter("glucose", self.cleaned_data)


class GlucoseFollowupForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = GlucoseFollowupFormValidator

    class Meta:
        model = GlucoseFollowup
        fields = "__all__"



from django.conf import settings
from django.views.generic import TemplateView
from edc_dashboard.view_mixins import EdcViewMixin
from edc_navbar import NavbarViewMixin


class HomeView(EdcViewMixin, NavbarViewMixin, TemplateView):
    template_name = f"{settings.APP_NAME}/bootstrap{settings.EDC_BOOTSTRAP}/home.html"
    navbar_name = "mocca_edc"
    navbar_selected_item = "home"



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import CrfStatusModelAdminMixin, crf_status_fieldset_tuple
from edc_model_admin import SimpleHistoryAdmin
from edc_model_admin.dashboard import (
    ModelAdminCrfDashboardMixin,
    ModelAdminSubjectDashboardMixin,
)


class ModelAdminMixin(ModelAdminSubjectDashboardMixin):
    pass


class CrfModelAdminMixin(CrfStatusModelAdminMixin, ModelAdminCrfDashboardMixin):

    pass


class CrfModelAdmin(ModelAdminCrfDashboardMixin, SimpleHistoryAdmin):

    pass


class GlucoseModelAdminMixin:
    form = None

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Blood Sugar Measurement",
            {
                "fields": (
                    "glucose_performed",
                    "glucose_fasted",
                    "glucose_date",
                    "glucose_value",
                    "glucose_quantifier",
                    "glucose_units",
                ),
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "crf_status": admin.VERTICAL,
        "glucose_performed": admin.VERTICAL,
        "glucose_fasted": admin.VERTICAL,
        "glucose_units": admin.VERTICAL,
    }



# Generated by Django 3.0.9 on 2020-12-15 00:41

import _socket
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_model_fields.fields.other_charfield
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0002_auto_20201215_0241"),
    ]

    operations = [
        migrations.CreateModel(
            name="CareStatus",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50, unique=True)),
                (
                    "care",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("unknown", "Unknown")],
                        max_length=25,
                        verbose_name="Is the patient in care?",
                    ),
                ),
                (
                    "care_not_in_reason",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="If not in care, why?",
                    ),
                ),
                (
                    "care_facility_location",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        help_text="Either integrated care or vertical care",
                        max_length=25,
                        verbose_name="Does the patient receive care in this facility",
                    ),
                ),
                (
                    "icc_since_mocca",
                    models.CharField(
                        choices=[
                            ("no_interruption", "Yes, without interruption"),
                            ("some_interruption", "Yes, with some interruption"),
                            (
                                "No",
                                "No, not since completing follow up with the MOCCA (original) trial.",
                            ),
                            ("unknown", "Unknown"),
                        ],
                        max_length=25,
                        verbose_name="Has the patient received integrated care since the leaving the MOCCA (orig) trial until now?",
                    ),
                ),
                (
                    "icc_since_mocca_comment",
                    edc_model_fields.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If some interruption in integrated care, please explain",
                    ),
                ),
                (
                    "icc",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        help_text="Either at this facility or elsewhere",
                        max_length=25,
                        verbose_name="Does the patient <u>currently</u> receive integrated care",
                    ),
                ),
                (
                    "icc_not_in_reason",
                    models.CharField(
                        choices=[
                            (
                                "icc_not_available",
                                "ICC not available (or closed) in this facility",
                            ),
                            ("moved", "Moved out of area"),
                            (
                                "dont_want",
                                "Personally chose not to continue with integrated care",
                            ),
                            (
                                "advised_to_vertical",
                                "Healthcare staff asked patient to return to vertical care",
                            ),
                            (
                                "referred_out",
                                "Referred to another facility without an ICC",
                            ),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="If not integrated care, why not?",
                    ),
                ),
                (
                    "care_comment",
                    models.TextField(
                        blank=True,
                        null=True,
                        verbose_name="Additional comments relevant to this patient's care",
                    ),
                ),
                (
                    "mocca_register",
                    models.OneToOneField(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="mocca_screening.MoccaRegister",
                        verbose_name="MOCCA (original) register details",
                    ),
                ),
            ],
            options={
                "verbose_name": "Care Status",
                "verbose_name_plural": "Care Status",
                "ordering": ("-modified", "-created"),
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
            },
        ),
    ]



# Generated by Django 3.1.6 on 2021-03-02 04:36

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_subject', '0015_auto_20210301_0745'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='drugsupplydm',
            name='drug',
        ),
        migrations.RemoveField(
            model_name='drugsupplydm',
            name='drug_refill',
        ),
        migrations.RemoveField(
            model_name='drugsupplyhiv',
            name='drug',
        ),
        migrations.RemoveField(
            model_name='drugsupplyhiv',
            name='drug_refill',
        ),
        migrations.RemoveField(
            model_name='drugsupplyhtn',
            name='drug',
        ),
        migrations.RemoveField(
            model_name='drugsupplyhtn',
            name='drug_refill',
        ),
        migrations.DeleteModel(
            name='DrugSupplyChol',
        ),
        migrations.DeleteModel(
            name='DrugSupplyDm',
        ),
        migrations.DeleteModel(
            name='DrugSupplyHiv',
        ),
        migrations.DeleteModel(
            name='DrugSupplyHtn',
        ),
    ]



from copy import copy

from edc_adverse_event.navbars import ae_navbar_item, tmg_navbar_item
from edc_data_manager.navbar_item import dm_navbar_item
from edc_lab_dashboard.navbars import navbar as lab_navbar
from edc_navbar import Navbar, site_navbars
from edc_review_dashboard.navbars import navbar as review_navbar

from mocca_dashboard.navbars import navbar as mocca_dashboard_navbar

navbar = Navbar(name="mocca_edc")

navbar_item = copy([item for item in lab_navbar.items if item.name == "specimens"][0])
navbar_item.active = False
navbar_item.label = "Specimens"
navbar.append_item(navbar_item)

navbar.append_item(
    [item for item in mocca_dashboard_navbar.items if item.name == "screened_subject"][0]
)

navbar.append_item(
    [item for item in mocca_dashboard_navbar.items if item.name == "consented_subject"][0]
)

for item in review_navbar.items:
    navbar.append_item(item)

navbar.append_item(tmg_navbar_item)
navbar.append_item(ae_navbar_item)
navbar.append_item(dm_navbar_item)

site_navbars.register(navbar)



from copy import copy

from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_action_item import action_fields, action_fieldset_tuple
from edc_form_label.form_label_modeladmin_mixin import FormLabelModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin

from ..admin_site import mocca_subject_admin
from ..forms import SubjectVisitMissedForm
from ..models import SubjectVisitMissed
from .modeladmin_mixins import CrfModelAdmin


@admin.register(SubjectVisitMissed, site=mocca_subject_admin)
class SubjectVisitMissedAdmin(CrfModelAdmin, FormLabelModelAdminMixin, SimpleHistoryAdmin):

    form = SubjectVisitMissedForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Contact History",
            {
                "fields": (
                    "survival_status",
                    "contact_attempted",
                    "contact_attempts_count",
                    "contact_made",
                    "contact_attempts_explained",
                    "contact_last_date",
                    "missed_reasons",
                    "missed_reasons_other",
                    "ltfu",
                    "comment",
                ),
            },
        ),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    filter_horizontal = ("missed_reasons",)

    radio_fields = {
        "survival_status": admin.VERTICAL,
        "contact_attempted": admin.VERTICAL,
        "contact_made": admin.VERTICAL,
        "ltfu": admin.VERTICAL,
    }

    def get_readonly_fields(self, request, obj=None):
        fields = super().get_readonly_fields(request, obj)
        action_flds = copy(list(action_fields))
        fields = list(action_flds) + list(fields)
        return fields



# Generated by Django 3.1.8 on 2021-04-27 21:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_ae', '0003_auto_20210225_0604'),
    ]

    operations = [
        migrations.AlterField(
            model_name='deathreport',
            name='narrative',
            field=models.TextField(null=True, verbose_name='Narrative'),
        ),
        migrations.AlterField(
            model_name='historicaldeathreport',
            name='narrative',
            field=models.TextField(null=True, verbose_name='Narrative'),
        ),
    ]



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple
from edc_form_label.form_label_modeladmin_mixin import FormLabelModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin

from ..admin_site import mocca_subject_admin
from ..forms import ComplicationsFollowupForm
from ..models import ComplicationsFollowup
from .modeladmin_mixins import CrfModelAdminMixin


@admin.register(ComplicationsFollowup, site=mocca_subject_admin)
class ComplicationsFollowupAdmin(
    CrfModelAdminMixin, FormLabelModelAdminMixin, SimpleHistoryAdmin
):

    form = ComplicationsFollowupForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Complications",
            {
                "fields": (
                    "stroke",
                    "stroke_date",
                    "heart_attack",
                    "heart_attack_date",
                    "renal_disease",
                    "renal_disease_date",
                    "vision",
                    "vision_date",
                    "numbness",
                    "numbness_date",
                    "foot_ulcers",
                    "foot_ulcers_date",
                    "complications",
                    "complications_other",
                )
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "crf_status": admin.VERTICAL,
        "heart_attack": admin.VERTICAL,
        "stroke": admin.VERTICAL,
        "renal_disease": admin.VERTICAL,
        "vision": admin.VERTICAL,
        "numbness": admin.VERTICAL,
        "foot_ulcers": admin.VERTICAL,
        "complications": admin.VERTICAL,
    }



from django.contrib import admin
from edc_model_admin import SimpleHistoryAdmin

from ..admin_site import mocca_prn_admin
from ..models import EndOfStudy
from .modeladmin_mixins import EndOfStudyModelAdminMixin


@admin.register(EndOfStudy, site=mocca_prn_admin)
class EndOfStudyAdmin(EndOfStudyModelAdminMixin, SimpleHistoryAdmin):

    pass



from edc_constants.constants import NO, NOT_APPLICABLE, YES
from edc_reportable.constants import GRADE4
from edc_utils.date import get_utcnow
from model_bakery.recipe import Recipe

from mocca_ae.models.ae_followup import AeFollowup
from mocca_ae.models.ae_initial import AeInitial
from mocca_ae.models.ae_susar import AeSusar
from mocca_ae.models.ae_tmg import AeTmg
from mocca_ae.models.death_report import DeathReport

aeinitial = Recipe(
    AeInitial,
    action_identifier=None,
    tracking_identifier=None,
    ae_description="A description of this event",
    ae_grade=GRADE4,
    ae_study_relation_possibility=YES,
    ae_start_date=get_utcnow().date(),
    ae_awareness_date=get_utcnow().date(),
    study_drug_relation="not_related",
    ae_treatment="Some special treatment",
    sae=NO,
    susar=NO,
    susar_reported=NOT_APPLICABLE,
    ae_cause=NO,
    ae_cause_other=None,
)

aetmg = Recipe(AeTmg, action_identifier=None, tracking_identifier=None)

aesusar = Recipe(AeSusar, action_identifier=None, tracking_identifier=None)

aefollowup = Recipe(
    AeFollowup, relevant_history=NO, action_identifier=None, tracking_identifier=None
)

deathreport = Recipe(
    DeathReport,
    subject_identifier=None,
    action_identifier=None,
    tracking_identifier=None,
)



from django.db import models
from edc_blood_results.model_mixins import GlucoseModelMixin
from edc_constants.choices import YES_NO, YES_NO_NA
from edc_constants.constants import NOT_APPLICABLE
from edc_lab.choices import RESULT_QUANTIFIER
from edc_lab.constants import EQ
from edc_model import models as edc_models
from edc_model.models import date_not_future

from ..model_mixins import CrfModelMixin


class Glucose(GlucoseModelMixin, CrfModelMixin, edc_models.BaseUuidModel):

    """See also proxy models"""

    glucose_performed = models.CharField(
        verbose_name=(
            "Has the patient had their glucose measured today or since the last visit?"
        ),
        max_length=15,
        choices=YES_NO,
    )

    glucose_date = models.DateField(
        validators=[date_not_future],
        null=True,
        blank=True,
    )

    glucose_fasted = models.CharField(
        verbose_name="Has the participant fasted?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    glucose_value = models.DecimalField(
        verbose_name="Glucose result",
        max_digits=8,
        decimal_places=2,
        null=True,
        blank=True,
    )

    glucose_quantifier = models.CharField(
        max_length=10,
        choices=RESULT_QUANTIFIER,
        default=EQ,
        null=True,
        blank=True,
    )

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Glucose"
        verbose_name_plural = "Glucose"



# Generated by Django 3.0.9 on 2020-12-15 12:52

import edc_sites.models
from django.db import migrations

import mocca_screening.models.subject_refusal_screening


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0006_auto_20201215_1543"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="subjectrefusalscreening",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", mocca_screening.models.subject_refusal_screening.Manager()),
            ],
        ),
    ]



from pprint import pprint

from django import forms
from django.test import TestCase
from edc_constants.constants import NOT_APPLICABLE, OTHER, STUDY_DEFINED_TIMEPOINT
from edc_form_validators import FormValidatorTestCaseMixin
from edc_utils import get_utcnow
from edc_visit_schedule.constants import DAY1
from edc_visit_tracking.constants import MISSED_VISIT, SCHEDULED

from mocca_lists.list_data import list_data
from mocca_lists.models import ClinicServices
from mocca_subject.choices import INFO_SOURCE
from mocca_subject.forms.subject_visit_form import SubjectVisitFormValidator

from ..mocca_test_case_mixin import MoccaTestCaseMixin


class TestSubjectVisit(MoccaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.subject_screening = self.get_subject_screening(report_datetime=get_utcnow())
        self.subject_consent = self.get_subject_consent(
            subject_screening=self.subject_screening
        )

    def test_baseline_subject_visit_form(self):
        appointment = self.get_appointment(
            subject_identifier=self.subject_consent.subject_identifier,
            visit_code=DAY1,
            visit_code_sequence=0,
            reason=SCHEDULED,
        )
        clinic_services = ClinicServices.objects.filter(name=STUDY_DEFINED_TIMEPOINT)
        cleaned_data = dict(
            appointment=appointment,
            report_datetime=appointment.appt_datetime,
            clinic_services=clinic_services,
            reason=SCHEDULED,
        )
        form_validator = SubjectVisitFormValidator(cleaned_data=cleaned_data)
        try:
            form_validator.validate()
        except forms.ValidationError:
            pass
        pprint(form_validator._errors)
        self.assertNotIn("health_services", form_validator._errors)


class TestSubjectVisitFormValidator(MoccaTestCaseMixin, FormValidatorTestCaseMixin, TestCase):

    form_validator_default_form_cls = SubjectVisitFormValidator

    def setUp(self):
        super().setUp()
        self.subject_screening = self.get_subject_screening(report_datetime=get_utcnow())
        self.subject_consent = self.get_subject_consent(
            subject_screening=self.subject_screening
        )
        self.subject_visit = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
            visit_code=DAY1,
        )

    def test_form_validator_allows_missed_visit(self):
        cleaned_data = {
            "appointment": self.subject_visit.appointment,
            "report_datetime": self.subject_visit.report_datetime,
            "reason": MISSED_VISIT,
            "clinic_services": ClinicServices.objects.filter(name=NOT_APPLICABLE),
            "info_source": NOT_APPLICABLE,
        }
        form_validator = self.validate_form_validator(cleaned_data)
        self.assertDictEqual({}, form_validator._errors)

    def test_missed_visit_raises_validation_error_if_clinic_services_not_not_applicable(
        self,
    ):
        cleaned_data = {
            "appointment": self.subject_visit.appointment,
            "report_datetime": self.subject_visit.report_datetime,
            "reason": MISSED_VISIT,
            "info_source": NOT_APPLICABLE,
        }
        for value, _ in list_data["mocca_lists.clinicservices"]:
            if value != NOT_APPLICABLE:
                with self.subTest(value=value):
                    cleaned_data.update(
                        {
                            "clinic_services": ClinicServices.objects.filter(name=value),
                        }
                    )
                    if value == OTHER:
                        cleaned_data.update(
                            {"clinic_services_other": "some other clinic service"}
                        )
                    else:
                        cleaned_data.pop("clinic_services_other", None)

                    form_validator = self.validate_form_validator(cleaned_data)
                    self.assertIn("clinic_services", form_validator._errors)
                    self.assertIn(
                        (
                            "Expected 'Not applicable' response "
                            "(if this is a missed visit report)."
                        ),
                        str(form_validator._errors.get("clinic_services")),
                    )
                    self.assertEqual(len(form_validator._errors), 1, form_validator._errors)

    def test_missed_visit_raises_validation_error_if_info_source_not_not_applicable(
        self,
    ):
        cleaned_data = {
            "appointment": self.subject_visit.appointment,
            "report_datetime": self.subject_visit.report_datetime,
            "reason": MISSED_VISIT,
            "clinic_services": ClinicServices.objects.filter(name=NOT_APPLICABLE),
        }

        for value, _ in INFO_SOURCE:
            if value != NOT_APPLICABLE:
                with self.subTest(value=value):
                    cleaned_data.update({"info_source": value})
                    if value == OTHER:
                        cleaned_data.update(
                            {"info_source_other": "some other information source"}
                        )
                    else:
                        cleaned_data.pop("info_source_other", None)

                    form_validator = self.validate_form_validator(cleaned_data)
                    self.assertIn("info_source", form_validator._errors)
                    self.assertIn(
                        "This field is not applicable.",
                        str(form_validator._errors.get("info_source")),
                    )
                    self.assertEqual(len(form_validator._errors), 1, form_validator._errors)

    def test_missed_visit_raises_validation_error_if_multiple_clinic_services_selected(self):
        cleaned_data = {
            "appointment": self.subject_visit.appointment,
            "report_datetime": self.subject_visit.report_datetime,
            "reason": MISSED_VISIT,
            "clinic_services": ClinicServices.objects.filter().exclude(name=OTHER),
            "info_source": NOT_APPLICABLE,
        }
        form_validator = self.validate_form_validator(cleaned_data)
        self.assertIn("clinic_services", form_validator._errors)
        self.assertIn(
            "Invalid combination. 'Not applicable' may not be combined with other selections",
            str(form_validator._errors.get("clinic_services")),
        )
        self.assertEqual(len(form_validator._errors), 1, form_validator._errors)



import sys

from django.apps import AppConfig as DjangoAppConfig
from django.apps import apps as django_apps
from django.core.management.color import color_style
from django.db.models.signals import post_migrate

from .sites import all_sites

style = color_style()


def post_migrate_update_sites(sender=None, **kwargs):
    from edc_sites.add_or_update_django_sites import add_or_update_django_sites

    sys.stdout.write(style.MIGRATE_HEADING("Updating sites:\n"))
    for country, sites in all_sites.items():
        add_or_update_django_sites(
            apps=django_apps,
            sites=sites,
            verbose=True,
        )
    sys.stdout.write("Done.\n")
    sys.stdout.flush()


class AppConfig(DjangoAppConfig):
    name = "mocca_sites"

    def ready(self):
        post_migrate.connect(post_migrate_update_sites, sender=self)



from django.db import models
from edc_model import models as edc_models

from ..model_mixins import CrfModelMixin
from .autocomplete_models import Rx
from .model_mixins import DrugRefillModelMixin


class DrugRefillHiv(DrugRefillModelMixin, CrfModelMixin, edc_models.BaseUuidModel):

    rx = models.ForeignKey(
        Rx,
        verbose_name="Which medicine did the patient receive today?",
        on_delete=models.PROTECT,
    )

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Drug Refill: HIV"
        verbose_name_plural = "Drug Refills: HIV"



# Generated by Django 3.0.9 on 2020-12-04 03:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0012_auto_20201204_0352"),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name="moccaregister",
            name="unique_first_name__last_name",
        ),
    ]
